@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gdevbit.c,v 1.9 2007/08/01 14:25:45 jemarch Exp $ */
+/* $Id: gdevbit.c,v 1.10 2007/09/11 15:24:12 Arabidopsis Exp $ */
 /* "Plain bits" devices to measure rendering time. */
 
 #include "math_.h"
@@ -269,6 +269,8 @@ const gx_device_printer gs_bitrgbtags_device =
         0 ,
         0 ,
         0 ,
+	0,
+	0,
         { 
             gx_default_install,
             gx_default_begin_page,
@@ -627,7 +629,7 @@ bit_print_page(gx_device_printer * pdev, FILE * prn_stream)
     int line_size = gdev_mem_bytes_per_scan_line((gx_device *) pdev);
     byte *in = gs_alloc_bytes(pdev->memory, line_size, "bit_print_page(in)");
     byte *data;
-    int nul = !strcmp(pdev->fname, "nul");
+    int nul = !strcmp(pdev->fname, "nul") || !strcmp(pdev->fname, "/dev/null");
     int lnum = 0, bottom = pdev->height;
 
     if (in == 0)
