@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -16,7 +16,7 @@
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 */
-/*$Id: gdevdevn.c,v 1.8 2007/08/01 14:25:47 jemarch Exp $ */
+/*$Id: gdevdevn.c,v 1.10 2007/09/11 15:23:57 Arabidopsis Exp $ */
 /* Example DeviceN process color model devices. */
 
 #include "math_.h"
@@ -31,6 +31,8 @@
 #include "gxdcconv.h"
 #include "gdevdevn.h"
 #include "gsequivc.h"
+#include "gxblend.h"
+#include "gdevp14.h"
 
 /*
  * Utility routines for common DeviceN related parameters:
@@ -355,6 +357,7 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
     bool num_spot_changed = false;
     int num_order = pdevn_params->num_separation_order_names;
     int max_sep = pdevn_params->max_separations;
+    int page_spot_colors = pdevn_params->page_spot_colors;
     gs_param_string_array scna;		/* SeparationColorNames array */
     gs_param_string_array sona;		/* SeparationOrder names array */
 
@@ -421,7 +424,8 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
 
 	    num_order = sona.size;
 	    for (i = 0; i < num_spot + npcmcolors; i++)
-		pdevn_params->separation_order_map[i] = GX_DEVICE_COLOR_MAX_COMPONENTS;
+		pdevn_params->separation_order_map[i] =
+		       			GX_DEVICE_COLOR_MAX_COMPONENTS;
 	    for (i = 0; i < num_order; i++) {
 	        /*
 	         * Check if names match either the process color model or
@@ -451,25 +455,29 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
             case 0:
 	        if (max_sep < 1 || max_sep > GX_DEVICE_COLOR_MAX_COMPONENTS)
 		    return_error(gs_error_rangecheck);
-#if !USE_COMPRESSED_ENCODING
-		/*
-		 * If we are not using a compressed pixel encoding, then the
-		 * device depth (pixel size) varies with the number of
-		 * separations.  The depth is constant with the compressed
-		 * encoding scheme.
-		 */
-	        {
-		    int depth =
-		        bpc_to_depth(max_sep, pdevn_params->bitspercomponent);
-
-                    if (depth > 8 * size_of(gx_color_index))
-		        return_error(gs_error_rangecheck);
-                    pdev->color_info.depth = depth;
-	        }
-#endif
-    	        pdevn_params->max_separations =
-		        pdev->color_info.max_components =
-    	                pdev->color_info.num_components = max_sep;
+        }
+        /*
+         * The PDF interpreter scans the resources for pages to try to
+	 * determine the number of spot colors.  (Unfortuneately there is
+	 * no way to determine the number of spot colors for a PS page
+	 * except to interpret the entire page.)  The spot color count for
+	 * a PDF page may be high since there may be spot colors in a PDF
+	 * page's resources that are not used.  However this does give us
+	 * an upper limit on the number of spot colors.  A value of -1
+	 * indicates that the number of spot colors in unknown (a PS file).
+         */
+        code = param_read_int(plist, param_name = "PageSpotColors",
+		       					&page_spot_colors);
+        switch (code) {
+            default:
+	        param_signal_error(plist, param_name, code);
+            case 1:
+		break;
+            case 0:
+	        if (page_spot_colors < -1)
+		    return_error(gs_error_rangecheck);
+	        if (page_spot_colors > GX_DEVICE_COLOR_MAX_COMPONENTS)
+		    page_spot_colors = GX_DEVICE_COLOR_MAX_COMPONENTS;
         }
         /* 
          * The DeviceN device can have zero components if nothing has been
@@ -483,10 +491,14 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
 	 * SeparationColorNames, SeparationOrder, or MaxSeparations.
 	 */
 	if (num_spot_changed || pdevn_params->max_separations != max_sep ||
-	    	    pdevn_params->num_separation_order_names != num_order) {
+	    	    pdevn_params->num_separation_order_names != num_order ||
+		    pdevn_params->page_spot_colors != page_spot_colors) {
 	    pdevn_params->separations.num_separations = num_spot;
 	    pdevn_params->num_separation_order_names = num_order;
     	    pdevn_params->max_separations = max_sep;
+    	    pdevn_params->page_spot_colors = page_spot_colors;
+	    if (max_sep != 0)
+		 pdev->color_info.max_components = max_sep;
 	    /*
 	     * If we have SeparationOrder specified then the number of
 	     * components is given by the number of names in the list.
@@ -495,10 +507,17 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
 	     * of ProcessColorModel components plus the number of
 	     * SeparationColorNames is used.
 	     */
-            pdev->color_info.num_components = (num_order) ? num_order 
+            pdev->color_info.num_components = (num_order)
+		? num_order 
 		: (pdevn_params->max_separations)
-				? pdevn_params->max_separations
-				: npcmcolors + num_spot;
+			? pdevn_params->max_separations
+			: (page_spot_colors >= 0)
+		       		? npcmcolors + num_spot + page_spot_colors
+	    			: pdev->color_info.max_components;
+            if (pdev->color_info.num_components >
+		    pdev->color_info.max_components)
+                pdev->color_info.num_components =
+		       	pdev->color_info.max_components;
 #if !USE_COMPRESSED_ENCODING
 	    /*
 	     * See earlier comment about the depth and non compressed
@@ -556,7 +575,15 @@ devn_printer_put_params(gx_device * pdev, gs_param_list * plist,
         /* Reset the sparable and linear shift, masks, bits. */
 	set_linear_color_bits_mask_shift(pdev);
     }
-
+    /*
+     * Also check for parameters which are being passed from the PDF 1.4
+     * compositior clist write device.  This device needs to pass info
+     * to the PDF 1.4 compositor clist reader device.  However this device
+     * is not crated until the clist is being read.  Thus we have to buffer
+     * this info in the output device.   (This is only needed for devices
+     * which support spot colors.)
+     */
+    code = pdf14_put_devn_params(pdev, pdevn_params, plist);
     return code;
 }
 
@@ -649,7 +676,7 @@ print_compressed_color_list(compressed_color_list_t * pcomp_list, int num_comp)
 /*
  * Allocate an list level element for our encode color list.
  */
-private compressed_color_list_t * 
+compressed_color_list_t * 
 alloc_compressed_color_list_elem(gs_memory_t * mem, int num_comps)
 {
     compressed_color_list_t * plist =
@@ -664,6 +691,43 @@ alloc_compressed_color_list_elem(gs_memory_t * mem, int num_comps)
     return plist;
 }
 
+/*
+ * Free the elements of a compressed color list.
+ */
+void
+free_compressed_color_list(gs_memory_t * mem,
+	       	compressed_color_list_t * pcomp_list)
+{
+    int i;
+
+    if (pcomp_list == NULL)
+	return;
+
+    /* Discard the sub levels. */
+    for (i = 0; i < pcomp_list->num_sub_level_ptrs; i++)
+       free_compressed_color_list(mem, pcomp_list->u.sub_level_ptrs[i]);
+
+    gs_free_object(mem, pcomp_list, "free_compressed_color_list");
+    return;
+}
+
+/*
+ * Free a set of separation names
+ */
+void
+free_separation_names(gs_memory_t * mem,
+	       	gs_separations * pseparation)
+{
+    int i;
+
+    /* Discard the sub levels. */
+    for (i = 0; i < pseparation->num_separations; i++)
+        gs_free_object(mem, pseparation->names[i].data,
+				"free_separation_names");
+    pseparation->num_separations = 0;
+    return;
+}
+
 /*
  * Add a new set of bit mapped colorant lists to our list of encoded color
  * colorants.
@@ -846,7 +910,7 @@ init_compressed_color_list(gs_memory_t *mem)
  * the number of colorants being used times size of the colorant value saved
  * must fit into a gx_color_index value.
  */
-static int num_comp_bits[MAX_ENCODED_COMPONENTS + 1] = {
+int num_comp_bits[MAX_ENCODED_COMPONENTS + 1] = {
 	8,	/* 0 colorants - not used */
 	8,	/* 1 colorants */
 	8,	/* 2 colorants */
@@ -874,7 +938,7 @@ static int num_comp_bits[MAX_ENCODED_COMPONENTS + 1] = {
 #define gx_color_value_factor(num_bits) \
     ((gx_max_color_value << 8) + 0xff) / ((1 << num_bits) - 1)
 
-static int comp_bit_factor[MAX_ENCODED_COMPONENTS + 1] = {
+int comp_bit_factor[MAX_ENCODED_COMPONENTS + 1] = {
 	gx_color_value_factor(8),		 /*  0 colorants (8 bits) */
 	gx_color_value_factor(8),		 /*  1 colorants (8 bits) */
 	gx_color_value_factor(8),		 /*  2 colorants (8 bits) */
@@ -1199,7 +1263,7 @@ devn_encode_compressed_color(gx_device *pdev, const gx_color_value colors[],
 /*
  * Find the bit map for given bit map index.
  */
-private comp_bit_map_list_t *
+comp_bit_map_list_t *
 find_bit_map(gx_color_index index, compressed_color_list_t * pcomp_list)
 {
     int loc = (int)(index >> (NUM_GX_COLOR_INDEX_BITS - 8));
@@ -1342,8 +1406,6 @@ devn_unpack_row(gx_device * dev, int num_comp, gs_devn_params * pdevn_params,
        		            *out++ = solid_color >> 8;
 		        else {
             	            *out++ = (factor * ((int)color & bit_mask)) >> 16;
-			    if (comp_num == 3 && out[-1] != 0)
-			        comp_num = comp_num;
 	    	            color >>= bit_count;
 		        }
 		    }
@@ -1527,6 +1589,7 @@ const spotcmyk_device gs_spotcmyk_device =
       DeviceCMYKComponents,	/* Names of color model colorants */
       4,			/* Number colorants for CMYK */
       0,			/* MaxSeparations has not been specified */
+      -1,			/* PageSpotColors has not been specified */
       {0},			/* SeparationNames */
       0,			/* SeparationOrder names */
       {0, 1, 2, 3, 4, 5, 6, 7 }	/* Initial component SeparationOrder */
@@ -1546,6 +1609,7 @@ const spotcmyk_device gs_devicen_device =
       NULL,			/* No names for standard DeviceN color model */
       0,			/* No standard colorants for DeviceN */
       0,			/* MaxSeparations has not been specified */
+      -1,			/* PageSpotColors has not been specified */
       {0},			/* SeparationNames */
       0,			/* SeparationOrder names */
       {0, 1, 2, 3, 4, 5, 6, 7 }	/* Initial component SeparationOrder */
@@ -1768,7 +1832,7 @@ repack_data(byte * source, byte * dest, int depth, int first_bit,
     return length;
 }
 
-private int write_pcx_file(gx_device_printer * pdev, char * filename, int ncomp,
+private int devn_write_pcx_file(gx_device_printer * pdev, char * filename, int ncomp,
 			    int bpc, int pcmlinelength);
 /* 
  * This is an example print page routine for a DeviceN device.  This routine
@@ -1875,14 +1939,14 @@ spotcmyk_print_page(gx_device_printer * pdev, FILE * prn_stream)
 
     /* Now convert the bit image files into PCX files */
     if (npcmcolors) {
-	code = write_pcx_file(pdev, (char *) &pdevn->fname,
+	code = devn_write_pcx_file(pdev, (char *) &pdevn->fname,
 				npcmcolors, bpc, pcmlinelength);
 	if (code < 0)
 	    return code;
     }
     for(i = 0; i < nspot; i++) {
 	sprintf(spotname, "%ss%d", pdevn->fname, i);
-	code = write_pcx_file(pdev, spotname, 1, bpc, linelength[i]);
+	code = devn_write_pcx_file(pdev, spotname, 1, bpc, linelength[i]);
 	if (code < 0)
 	    return code;
     }
@@ -1966,8 +2030,8 @@ private const pcx_header pcx_header_prototype =
 
 
 /* Forward declarations */
-private void pcx_write_rle(const byte *, const byte *, int, FILE *);
-private int pcx_write_page(gx_device_printer * pdev, FILE * infile,
+private void devn_pcx_write_rle(const byte *, const byte *, int, FILE *);
+private int devn_pcx_write_page(gx_device_printer * pdev, FILE * infile,
     int linesize, FILE * outfile, pcx_header * phdr, bool planar, int depth);
 
 static const byte pcx_cmyk_palette[16 * 3] =
@@ -2002,7 +2066,7 @@ static const byte pcx_ega_palette[16 * 3] =
  *   num_planes - The number of planes.
  */
 private bool
-setup_pcx_header(gx_device_printer * pdev, pcx_header * phdr, int num_planes, int bits_per_plane)
+devn_setup_pcx_header(gx_device_printer * pdev, pcx_header * phdr, int num_planes, int bits_per_plane)
 {
     bool planar = true; /* Invalid cases could cause an indeterminizm. */
  
@@ -2130,7 +2194,7 @@ pc_write_mono_palette(gx_device * dev, uint max_index, FILE * file)
  *   num_planes - The number of planes.
  */
 private int
-finish_pcx_file(gx_device_printer * pdev, FILE * file, pcx_header * header, int num_planes, int bits_per_plane)
+devn_finish_pcx_file(gx_device_printer * pdev, FILE * file, pcx_header * header, int num_planes, int bits_per_plane)
 {
     switch (num_planes) {
         case 1:
@@ -2204,7 +2268,7 @@ finish_pcx_file(gx_device_printer * pdev, FILE * file, pcx_header * header, int
 
 /* Send the page to the printer. */
 private int
-write_pcx_file(gx_device_printer * pdev, char * filename, int ncomp,
+devn_write_pcx_file(gx_device_printer * pdev, char * filename, int ncomp,
 			    int bpc, int linesize)
 {
     pcx_header header;
@@ -2225,10 +2289,10 @@ write_pcx_file(gx_device_printer * pdev, char * filename, int ncomp,
 	return_error(gs_error_invalidfileaccess);
     }
 
-    planar = setup_pcx_header(pdev, &header, ncomp, bpc);
-    code = pcx_write_page(pdev, in, linesize, out, &header, planar, depth);
+    planar = devn_setup_pcx_header(pdev, &header, ncomp, bpc);
+    code = devn_pcx_write_page(pdev, in, linesize, out, &header, planar, depth);
     if (code >= 0)
-        code = finish_pcx_file(pdev, out, &header, ncomp, bpc);
+        code = devn_finish_pcx_file(pdev, out, &header, ncomp, bpc);
 
     fclose(in);
     fclose(out);
@@ -2239,7 +2303,7 @@ write_pcx_file(gx_device_printer * pdev, char * filename, int ncomp,
 /* This routine is used for all formats. */
 /* The caller has set header->bpp, nplanes, and palette. */
 private int
-pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE * outfile,
+devn_pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE * outfile,
 	       pcx_header * phdr, bool planar, int depth)
 {
     int raster = linesize;
@@ -2283,7 +2347,7 @@ pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE * out
 		*end = end[-1];
 		++end;
 	    }
-	    pcx_write_rle(row, end, 1, outfile);
+	    devn_pcx_write_rle(row, end, 1, outfile);
 	} else
 	    switch (depth) {
 
@@ -2313,7 +2377,7 @@ pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE * out
 			    /* We might be one byte short of rsize. */
 			    if (to < pend)
 				*to = to[-1];
-			    pcx_write_rle(plane, pend, 1, outfile);
+			    devn_pcx_write_rle(plane, pend, 1, outfile);
 			}
 		    }
 		    break;
@@ -2323,7 +2387,7 @@ pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE * out
 			int pnum;
 
 			for (pnum = 0; pnum < 3; ++pnum) {
-			    pcx_write_rle(row + pnum, row + raster, 3, outfile);
+			    devn_pcx_write_rle(row + pnum, row + raster, 3, outfile);
 			    if (pdev->width & 1)
 				fputc(0, outfile);		/* pad to even */
 			}
@@ -2348,7 +2412,7 @@ pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE * out
 
 /* Write one line in PCX run-length-encoded format. */
 private void
-pcx_write_rle(const byte * from, const byte * end, int step, FILE * file)
+devn_pcx_write_rle(const byte * from, const byte * end, int step, FILE * file)
 {  /*
     * The PCX format theoretically allows encoding runs of 63
     * identical bytes, but some readers can't handle repetition
