@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 /* gdevdsp.c */
-/* $Id: gdevdsp.c,v 1.9 2007/08/01 14:25:47 jemarch Exp $ */
+/* $Id: gdevdsp.c,v 1.11 2007/09/11 15:23:44 Arabidopsis Exp $ */
 
 /*
  * DLL based display device driver.
@@ -95,6 +95,7 @@ private dev_proc_get_color_comp_index(display_separation_get_color_comp_index);
 private dev_proc_encode_color(display_separation_encode_color);
 private dev_proc_decode_color(display_separation_decode_color);
 private dev_proc_update_spot_equivalent_colors(display_update_spot_equivalent_colors);
+private dev_proc_ret_devn_params(display_ret_devn_params);
 
 
 private const gx_device_procs display_procs =
@@ -159,7 +160,8 @@ private const gx_device_procs display_procs =
     NULL,				/* fill_linear_color_scanline */\
     NULL,				/* fill_linear_color_trapezoid */\
     NULL,				/* fill_linear_color_triangle */\
-    display_update_spot_equivalent_colors /* update_spot_equivalent_colors */
+    display_update_spot_equivalent_colors, /* update_spot_equivalent_colors */
+    display_ret_devn_params		/* ret_devn_params */\
 };
 
 /* GC descriptor */
@@ -213,6 +215,7 @@ const gx_device_display gs_display_device =
       DeviceCMYKComponents,     /* Names of color model colorants */
       4,                        /* Number of colorants for CMYK */
       0,                        /* MaxSeparations has not been specified */
+      -1,			/* PageSpotColors has not been specified */
       {0},                      /* SeparationNames */
       {0},                      /* SeparationOrder names */
       {0, 1, 2, 3, 4, 5, 6, 7 } /* Initial component SeparationOrder */
@@ -1171,6 +1174,17 @@ display_update_spot_equivalent_colors(gx_device * dev, const gs_state * pgs)
     return 0;
 }
 
+/*
+ *  Device proc for returning a pointer to DeviceN parameter structure
+ */
+private gs_devn_params *
+display_ret_devn_params(gx_device * dev)
+{
+    gx_device_display * pdev = (gx_device_display *)dev;
+
+    return &pdev->devn_params;
+}
+
 /*
  * This routine will check to see if the color component name  match those
  * that are available amoung the current device's color components.  
@@ -1343,8 +1357,9 @@ display_alloc_bitmap(gx_device_display * ddev, gx_device * param_dev)
      * so we can place the bitmap in special memory.
      */
     ddev->mdev->line_pointer_memory = ddev->mdev->memory;
-    ddev->ulBitmapSize = gdev_mem_bits_size(ddev->mdev,
-	ddev->mdev->width, ddev->mdev->height);
+    if (gdev_mem_bits_size(ddev->mdev, ddev->mdev->width, ddev->mdev->height,
+	&(ddev->ulBitmapSize)) < 0)
+	return_error(gs_error_VMerror);
 
     /* allocate bitmap using an allocator not subject to GC */
     if (ddev->callback->display_memalloc 
