@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -16,7 +16,7 @@
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 */
-/* $Id: gdevnfwd.c,v 1.9 2007/08/01 14:25:50 jemarch Exp $ */
+/* $Id: gdevnfwd.c,v 1.11 2007/09/11 15:24:07 Arabidopsis Exp $ */
 /* Null and forwarding device implementation */
 #include "gx.h"
 #include "gserrors.h"
@@ -110,6 +110,7 @@ gx_device_forward_fill_in_procs(register gx_device_forward * dev)
     fill_dev_proc(dev, fill_linear_color_trapezoid, gx_forward_fill_linear_color_trapezoid);
     fill_dev_proc(dev, fill_linear_color_triangle, gx_forward_fill_linear_color_triangle);
     fill_dev_proc(dev, update_spot_equivalent_colors, gx_forward_update_spot_equivalent_colors);
+    fill_dev_proc(dev, ret_devn_params, gx_forward_ret_devn_params);
     gx_device_fill_in_procs((gx_device *) dev);
 }
 
@@ -756,10 +757,19 @@ gx_forward_pattern_manage(gx_device * dev, gx_bitmap_id id,
 
     /* Note that clist sets fdev->target == fdev, 
        so this function is unapplicable to clist. */
-    if (tdev == 0)
+    if (tdev == 0) {
+	if (function == pattern_manage__shfill_doesnt_need_path) {
+	    if (dev->procs.fill_path == gx_default_fill_path)
+		return 1;
+	}
 	return 0;
-    else
+    } else {
+	if (function == pattern_manage__handles_clip_path) {
+	    if (dev->procs.fill_path == gx_default_fill_path)
+		return 0;
+	}
 	return dev_proc(tdev, pattern_manage)(tdev, id, pinst, function);
+    }
 }
 
 int
@@ -849,6 +859,17 @@ gx_forward_update_spot_equivalent_colors(gx_device *dev, const gs_state * pgs)
     return code;
 }
 
+gs_devn_params * 
+gx_forward_ret_devn_params(gx_device *dev)
+{
+    gx_device_forward * const fdev = (gx_device_forward *)dev;
+    gx_device *tdev = fdev->target;
+
+    if (tdev != NULL)
+	return dev_proc(tdev, ret_devn_params)(tdev);
+    return NULL;
+}
+
 
 /* ---------------- The null device(s) ---------------- */
 
