@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gdevpdfp.c,v 1.9 2007/08/01 14:25:52 jemarch Exp $ */
+/* $Id: gdevpdfp.c,v 1.11 2007/09/11 15:24:04 Arabidopsis Exp $ */
 /* Get/put parameters for PDF-writing driver */
 #include "memory_.h"
 #include "string_.h"
@@ -89,7 +89,7 @@ private const gs_param_item_t pdf_param_items[] = {
     pi("NoEncrypt", gs_param_type_string, NoEncrypt),
 
 	/* Target viewer capabilities (Ghostscript-specific)  */
-    pi("ForOPDFRead", gs_param_type_bool, ForOPDFRead),
+ /* pi("ForOPDFRead", gs_param_type_bool, ForOPDFRead),			    pdfwrite-only */
     pi("PatternImagemask", gs_param_type_bool, PatternImagemask),
     pi("MaxClipPathSize", gs_param_type_int, MaxClipPathSize),
     pi("MaxShadingBitmapSize", gs_param_type_int, MaxShadingBitmapSize),
@@ -97,7 +97,7 @@ private const gs_param_item_t pdf_param_items[] = {
     pi("HaveTrueTypes", gs_param_type_bool, HaveTrueTypes),
     pi("HaveCIDSystem", gs_param_type_bool, HaveCIDSystem),
     pi("HaveTransparency", gs_param_type_bool, HaveTransparency),
-    pi("OPDFReadProcsetPath", gs_param_type_string, OPDFReadProcsetPath),
+ /* pi("OPDFReadProcsetPath", gs_param_type_string, OPDFReadProcsetPath),   ps2write-only */
     pi("CompressEntireFile", gs_param_type_bool, CompressEntireFile),
     pi("PDFX", gs_param_type_bool, PDFX),
     pi("PDFA", gs_param_type_bool, PDFA),
@@ -202,6 +202,8 @@ gdev_pdf_get_params(gx_device * dev, gs_param_list * plist)
 	(code = param_write_int(plist, ".EmbedFontObjects", &EmbedFontObjects)) < 0 ||
 	(code = param_write_int(plist, "CoreDistVersion", &cdv)) < 0 ||
 	(code = param_write_float(plist, "CompatibilityLevel", &cl)) < 0 ||
+	(pdev->is_ps2write && (code = param_write_string(plist, "OPDFReadProcsetPath", &pdev->OPDFReadProcsetPath)) < 0) ||
+	(!pdev->is_ps2write && (code = param_write_bool(plist, "ForOPDFRead", &pdev->ForOPDFRead)) < 0) ||
 	/* Indicate that we can process pdfmark and DSC. */
 	(param_requested(plist, "pdfmark") > 0 &&
 	 (code = param_write_null(plist, "pdfmark")) < 0) ||
@@ -214,19 +216,17 @@ gdev_pdf_get_params(gx_device * dev, gs_param_list * plist)
 
 /* ---------------- Put parameters ---------------- */
 
-/* Put parameters. */
-int
-gdev_pdf_put_params(gx_device * dev, gs_param_list * plist)
+/* Put parameters, implementation */
+private int
+gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_param_list * plist)
 {
-    gx_device_pdf *pdev = (gx_device_pdf *) dev;
     int ecode, code;
-    gx_device_pdf save_dev;
+    gx_device_pdf *pdev = (gx_device_pdf *) dev;
     float cl = (float)pdev->CompatibilityLevel;
     bool locked = pdev->params.LockDistillerParams;
     gs_param_name param_name;
     enum psdf_color_conversion_strategy save_ccs = pdev->params.ColorConversionStrategy;
   
-
     pdev->pdf_memory = gs_memory_stable(pdev->memory);
     /*
      * If this is a pseudo-parameter (pdfmark or DSC),
@@ -298,8 +298,6 @@ gdev_pdf_put_params(gx_device * dev, gs_param_list * plist)
 		param_signal_error(plist, param_name, ecode = gs_error_rangecheck);
 	}
 
-	save_dev = *pdev;
-
 	switch (code = param_read_float(plist, (param_name = "CompatibilityLevel"), &cl)) {
 	    default:
 		ecode = code;
@@ -338,6 +336,10 @@ gdev_pdf_put_params(gx_device * dev, gs_param_list * plist)
 
 	    plist->memory = pdev->pdf_memory;
 	    code = gs_param_read_items(plist, pdev, pdf_param_items);
+	    if (code < 0 ||
+		(pdev->is_ps2write && (code = param_read_string(plist, "OPDFReadProcsetPath", &pdev->OPDFReadProcsetPath)) < 0) ||
+		(!pdev->is_ps2write && (code = param_read_bool(plist, "ForOPDFRead", &pdev->ForOPDFRead)) < 0)
+		);
 	    plist->memory = mem;
 	}
 	if (code < 0)
@@ -350,11 +352,11 @@ gdev_pdf_put_params(gx_device * dev, gs_param_list * plist)
 	     */
 	    long fon = pdev->FirstObjectNumber;
 
-	    if (fon != save_dev.FirstObjectNumber) {
+	    if (fon != save_dev->FirstObjectNumber) {
 		if (fon <= 0 || fon > 0x7fff0000 ||
 		    (pdev->next_id != 0 &&
 		     pdev->next_id !=
-		     save_dev.FirstObjectNumber + pdf_num_initial_ids)
+		     save_dev->FirstObjectNumber + pdf_num_initial_ids)
 		    ) {
 		    ecode = gs_error_rangecheck;
 		    param_signal_error(plist, "FirstObjectNumber", ecode);
@@ -440,7 +442,7 @@ gdev_pdf_put_params(gx_device * dev, gs_param_list * plist)
     }
     if (pdev->params.ColorConversionStrategy == ccs_UseDeviceIndependentColor) {
 	if (!pdev->UseCIEColor) {
-	    eprintf("Set UseCUEColor for UseDeviceIndependentColor to work properly.\n");
+	    eprintf("Set UseCIEColor for UseDeviceIndependentColor to work properly.\n");
 	    ecode = gs_note_error(gs_error_rangecheck);
 	    pdev->UseCIEColor = true;
 	}
@@ -450,7 +452,7 @@ gdev_pdf_put_params(gx_device * dev, gs_param_list * plist)
 	    eprintf("UseDeviceDependentColorForImages is not supported. Use UseDeviceIndependentColor.\n");
 	    pdev->params.ColorConversionStrategy = ccs_UseDeviceIndependentColor;
 	    if (!pdev->UseCIEColor) {
-		eprintf("Set UseCUEColor for UseDeviceIndependentColor to work properly.\n");
+		eprintf("Set UseCIEColor for UseDeviceIndependentColor to work properly.\n");
 		ecode = gs_note_error(gs_error_rangecheck);
 		pdev->UseCIEColor = true;
 	    }
@@ -505,7 +507,7 @@ gdev_pdf_put_params(gx_device * dev, gs_param_list * plist)
 				 dev->HWResolution[1] / factor);
     }
 #undef MAX_EXTENT
-    if (pdev->FirstObjectNumber != save_dev.FirstObjectNumber) {
+    if (pdev->FirstObjectNumber != save_dev->FirstObjectNumber) {
 	if (pdev->xref.file != 0) {
 	    fseek(pdev->xref.file, 0L, SEEK_SET);
 	    pdf_initialize_ids(pdev);
@@ -516,21 +518,41 @@ gdev_pdf_put_params(gx_device * dev, gs_param_list * plist)
     return 0;
  fail:
     /* Restore all the parameters to their original state. */
-    pdev->version = save_dev.version;
-    pdf_set_process_color_model(pdev, save_dev.pcm_color_info_index);
-    pdev->saved_fill_color = save_dev.saved_fill_color;
-    pdev->saved_stroke_color = save_dev.saved_fill_color;
+    pdev->version = save_dev->version;
+    pdf_set_process_color_model(pdev, save_dev->pcm_color_info_index);
+    pdev->saved_fill_color = save_dev->saved_fill_color;
+    pdev->saved_stroke_color = save_dev->saved_fill_color;
     {
 	const gs_param_item_t *ppi = pdf_param_items;
 
 	for (; ppi->key; ++ppi)
 	    memcpy((char *)pdev + ppi->offset,
-		   (char *)&save_dev + ppi->offset,
+		   (char *)save_dev + ppi->offset,
 		   gs_param_type_sizes[ppi->type]);
+	pdev->ForOPDFRead = save_dev->ForOPDFRead;
+	pdev->OPDFReadProcsetPath = save_dev->OPDFReadProcsetPath;
     }
     return ecode;
 }
 
+/* Put parameters */
+int
+gdev_pdf_put_params(gx_device * dev, gs_param_list * plist)
+{
+    int code;
+    gx_device_pdf *pdev = (gx_device_pdf *) dev;
+    gs_memory_t *mem = gs_memory_stable(pdev->memory);
+    gx_device_pdf *save_dev = gs_malloc(mem, sizeof(gx_device_pdf), 1,
+        "saved gx_device_pdf");
+
+    if (!save_dev)
+        return_error(gs_error_VMerror);
+    memcpy(save_dev, pdev, sizeof(gx_device_pdf));
+    code = gdev_pdf_put_params_impl(dev, save_dev, plist);
+    gs_free(mem, save_dev, sizeof(gx_device_pdf), 1, "saved gx_device_pdf");
+    return code;
+}
+
 /* ---------------- Process DSC comments ---------------- */
 
 private int
