@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gdevpdtc.c,v 1.8 2007/08/01 14:25:52 jemarch Exp $ */
+/* $Id: gdevpdtc.c,v 1.10 2007/09/11 15:24:15 Arabidopsis Exp $ */
 /* Composite and CID-based text processing for pdfwrite. */
 #include "memory_.h"
 #include "string_.h"
@@ -141,11 +141,24 @@ process_composite_text(gs_text_enum_t *pte, void *vbuf, uint bsize)
 	    out.text.space.s_char = space_char;
 
 	    gs_matrix_multiply(&prev_font->FontMatrix, psmat, &fmat);
+	    out.index = 0; /* Note : we don't reset out.xy_index here. */
 	    code = pdf_process_string_aux(&out, &str, NULL, &fmat, &text_state);
 	    if (code < 0)
 		return code;
 	    curr.xy_index = out.xy_index; /* pdf_encode_process_string advanced it. */
-	    gs_text_enum_copy_dynamic(pte, (gs_text_enum_t *)&prev, true);
+	    if (out.index < str.size) {
+		gs_glyph glyph;
+
+		/* Advance *pte exactly for out.index chars, 
+		   because above we stored bytes into buf. */
+		while (out.index--)
+		    pte->orig_font->procs.next_char_glyph(pte, &chr, &glyph);
+		font_code = 2; /* force exiting the loop */
+	    } else {
+		/* advance *pte past the current substring */
+		gs_text_enum_copy_dynamic(pte, (gs_text_enum_t *)&prev, true);
+	    }
+	    pte->xy_index = out.xy_index;
 	    if (return_width) {
 		pte->returned.total_width.x = total_width.x +=
 		    out.returned.total_width.x;
