@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gdevprn.c,v 1.9 2007/08/01 14:25:54 jemarch Exp $ */
+/* $Id: gdevprn.c,v 1.11 2007/09/11 15:24:19 Arabidopsis Exp $ */
 /* Generic printer driver support */
 #include "ctype_.h"
 #include "gdevprn.h"
@@ -143,6 +143,7 @@ BACKTRACE(pdev);
 open_c:
     ppdev->buf = base;
     ppdev->buffer_space = space;
+    clist_init_io_procs(pclist_dev, false);
     clist_init_params(pclist_dev, base, space, pdev,
 		      ppdev->printer_procs.buf_procs,
 		      space_params->band, ppdev->is_async_renderer,
@@ -203,6 +204,13 @@ gdev_prn_tear_down(gx_device *pdev, byte **the_memory)
 	ppdev->buf = 0;
 	ppdev->buffer_space = 0;
 	is_command_list = true;
+	
+	/* If the clist is a reader clist, free any band_complexity_array
+	 * memory used by same.
+	 */
+	if (!CLIST_IS_WRITER(pclist_dev))
+	    gx_clist_reader_free_band_complexity_array(pclist_dev);
+	
     } else {
 	/* point at the device bitmap, no need to close mem dev */
 	*the_memory = pmemdev->base;
@@ -230,6 +238,7 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
     int save_height = 0x0badf00d; /* Quiet compiler */
     bool is_command_list = false; /* Quiet compiler */
     bool save_is_command_list = false; /* Quiet compiler */
+    bool size_ok = 0;
     int ecode = 0;
     int pass;
     gs_memory_t *buffer_memory =
@@ -272,11 +281,16 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 
 	/* Init clist/mem device-specific fields */
 	memset(ppdev->skip, 0, sizeof(ppdev->skip));
-	ppdev->printer_procs.buf_procs.size_buf_device
-	    (&buf_space, pdev, NULL, pdev->height, false);
-	if (ppdev->page_uses_transparency)
-	    pdf14_trans_buffer_size = new_height
-		* (ESTIMATED_PDF14_ROW_SPACE(new_width) >> 3);
+	size_ok = ppdev->printer_procs.buf_procs.size_buf_device
+	    (&buf_space, pdev, NULL, pdev->height, false) >= 0;
+	if (ppdev->page_uses_transparency) 
+	    if (new_height < max_ulong/(ESTIMATED_PDF14_ROW_SPACE(new_width) >> 3))
+		pdf14_trans_buffer_size = new_height
+		    * (ESTIMATED_PDF14_ROW_SPACE(new_width) >> 3);
+	    else {
+		size_ok = 0;
+		pdf14_trans_buffer_size = 0;
+	    }
 	mem_space = buf_space.bits + buf_space.line_ptrs
 		    + pdf14_trans_buffer_size;
 
@@ -302,7 +316,7 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 	else {
 	    is_command_list = space_params.banding_type == BandingAlways ||
 		mem_space >= space_params.MaxBitmap ||
-		mem_space != (uint)mem_space;	    /* too big to allocate */
+		!size_ok;	    /* too big to allocate */
 	}
 	if (!is_command_list) {
 	    /* Try to allocate memory for full memory buffer */
@@ -357,7 +371,7 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 	    ppdev->buffer_space = 0;
 	    if ((code = gdev_create_buf_device
 		 (ppdev->printer_procs.buf_procs.create_buf_device,
-		  &bdev, pdev, NULL, NULL, false)) < 0 ||
+		  &bdev, pdev, NULL, NULL, NULL)) < 0 ||
 		(code = ppdev->printer_procs.buf_procs.setup_buf_device
 		 (bdev, base, buf_space.raster,
 		  (byte **)(base + buf_space.bits), 0, pdev->height,
@@ -401,12 +415,8 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 	COPY_PROC(get_color_comp_index);
 	COPY_PROC(encode_color);
 	COPY_PROC(decode_color);
-	COPY_PROC(begin_image);
-	COPY_PROC(text_begin);
-	COPY_PROC(fill_path);
-	COPY_PROC(stroke_path);
-	COPY_PROC(fill_rectangle_hl_color);
 	COPY_PROC(update_spot_equivalent_colors);
+	COPY_PROC(ret_devn_params);
 #undef COPY_PROC
 	/* If using a command list, already opened the device. */
 	if (is_command_list)
@@ -973,9 +983,9 @@ int
 gdev_create_buf_device(create_buf_device_proc_t cbd_proc, gx_device **pbdev,
 		       gx_device *target,
 		       const gx_render_plane_t *render_plane,
-		       gs_memory_t *mem, bool for_band)
+		       gs_memory_t *mem, gx_band_complexity_t *band_complexity)
 {
-    int code = cbd_proc(pbdev, target, render_plane, mem, for_band);
+    int code = cbd_proc(pbdev, target, render_plane, mem, band_complexity);
 
     if (code < 0)
 	return code;
@@ -990,7 +1000,7 @@ gdev_create_buf_device(create_buf_device_proc_t cbd_proc, gx_device **pbdev,
  */
 int
 gx_default_create_buf_device(gx_device **pbdev, gx_device *target,
-    const gx_render_plane_t *render_plane, gs_memory_t *mem, bool for_band)
+    const gx_render_plane_t *render_plane, gs_memory_t *mem, gx_band_complexity_t *band_complexity)
 {
     int plane_index = (render_plane ? render_plane->index : -1);
     int depth;
@@ -1019,7 +1029,7 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target,
         check_device_separable((gx_device *)mdev);
 	gx_device_fill_in_procs((gx_device *)mdev);
     } else
-	gs_make_mem_device(mdev, mdproto, mem, (for_band ? 1 : 0),
+	gs_make_mem_device(mdev, mdproto, mem, (band_complexity == NULL ? 1 : 0),
 			   (target == (gx_device *)mdev ? NULL : target));
     mdev->width = target->width;
     /*
@@ -1055,16 +1065,20 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target,
 int
 gx_default_size_buf_device(gx_device_buf_space_t *space, gx_device *target,
 			   const gx_render_plane_t *render_plane,
-			   int height, bool for_band)
+			   int height, bool not_used)
 {
     gx_device_memory mdev;
 
+    space->line_ptrs = 0;	/*				*/
+    space->bits = 0;		/* clear in case of failure	*/
+    space->raster = 0;		/*				*/
     mdev.color_info.depth =
 	(render_plane && render_plane->index >= 0 ? render_plane->depth :
 	 target->color_info.depth);
     mdev.width = target->width;
     mdev.num_planes = 0;
-    space->bits = gdev_mem_bits_size(&mdev, target->width, height);
+    if (gdev_mem_bits_size(&mdev, target->width, height, &(space->bits)) < 0)
+	return_error(gs_error_VMerror);
     space->line_ptrs = gdev_mem_line_ptrs_size(&mdev, target->width, height);
     space->raster = gdev_mem_raster(&mdev);
     return 0;
