@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gdevpxen.h,v 1.8 2007/08/01 14:25:55 jemarch Exp $ */
+/* $Id: gdevpxen.h,v 1.10 2007/09/11 15:24:32 Arabidopsis Exp $ */
 /* Enumerated attribute value definitions for PCL XL */
 
 #ifndef gdevpxen_INCLUDED
@@ -313,11 +313,12 @@ typedef enum {
 
 typedef enum {
     eNoTreatment = 0,
-    eScreenMatch,
     eVivid,
-    eUnknown1,
-    eUnknown2,
-    eUnknown3,
+    eScreenMatch,
+    eText,
+    eOutofGamut,
+    eCIELabmatch1,
+    eCIELabmatch2,
     pxeColorTreatment_next
 } pxeColorTreatment;
     
