@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gdevrops.c,v 1.8 2007/08/01 14:25:55 jemarch Exp $ */
+/* $Id: gdevrops.c,v 1.10 2007/09/11 15:24:09 Arabidopsis Exp $ */
 /* RasterOp source device */
 #include "gx.h"
 #include "gserrors.h"
@@ -111,7 +111,8 @@ private const gx_device_rop_texture gs_rop_texture_device = {
      gx_forward_fill_linear_color_scanline,
      gx_forward_fill_linear_color_trapezoid,
      gx_forward_fill_linear_color_triangle,
-     gx_forward_update_spot_equivalent_colors
+     gx_forward_update_spot_equivalent_colors,
+     gx_forward_ret_devn_params
     },
     0,				/* target */
     lop_default			/* log_op */
