@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gdevtsep.c,v 1.4 2007/08/01 14:25:57 jemarch Exp $ */
+/* $Id: gdevtsep.c,v 1.6 2007/09/11 15:23:44 Arabidopsis Exp $ */
 /* tiffgray device:  8-bit Gray uncompressed TIFF device */
 /* tiff32nc device:  32-bit CMYK uncompressed TIFF device */
 /* tiffsep device: Generate individual TIFF gray files for each separation. */
@@ -244,6 +244,7 @@ private dev_proc_decode_color(tiffsep_decode_color);
 private dev_proc_encode_color(tiffsep_encode_compressed_color);
 private dev_proc_decode_color(tiffsep_decode_compressed_color);
 private dev_proc_update_spot_equivalent_colors(tiffsep_update_spot_equivalent_colors);
+private dev_proc_ret_devn_params(tiffsep_ret_devn_params);
 
 
 /*
@@ -368,7 +369,8 @@ gs_private_st_composite_final(st_tiffsep_device, tiffsep_device,
 	NULL,				/* fill_linear_color_scanline */\
 	NULL,				/* fill_linear_color_trapezoid */\
 	NULL,				/* fill_linear_color_triangle */\
-	tiffsep_update_spot_equivalent_colors /* update_spot_equivalent_colors */\
+	tiffsep_update_spot_equivalent_colors, /* update_spot_equivalent_colors */\
+	tiffsep_ret_devn_params		/* ret_devn_params */\
 }
 
 #define tiffsep_device_body(procs, dname, ncomp, pol, depth, mg, mc, sl, cn)\
@@ -427,6 +429,7 @@ const tiffsep_device gs_tiffsep_device =
       DeviceCMYKComponents,	/* Names of color model colorants */
       4,			/* Number colorants for CMYK */
       0,			/* MaxSeparations has not been specified */
+      -1,			/* PageSpotColors has not been specified */
       {0},			/* SeparationNames */
       0,			/* SeparationOrder names */
       {0, 1, 2, 3, 4, 5, 6, 7 }	/* Initial component SeparationOrder */
@@ -561,6 +564,17 @@ tiffsep_update_spot_equivalent_colors(gx_device * dev, const gs_state * pgs)
     return 0;
 }
 
+/*
+ *  Device proc for returning a pointer to DeviceN parameter structure
+ */
+private gs_devn_params *
+tiffsep_ret_devn_params(gx_device * dev)
+{
+    tiffsep_device * pdev = (tiffsep_device *)dev;
+
+    return &pdev->devn_params;
+}
+
 /* Get parameters.  We provide a default CRD. */
 private int
 tiffsep_get_params(gx_device * pdev, gs_param_list * plist)
@@ -1063,7 +1077,7 @@ tiffsep_print_page(gx_device_printer * pdev, FILE * file)
 	gs_free_object(pdev->memory, sep_line, "tiffsep_print_page");
     }
 
-#if DEBUG && 0
+#if defined(DEBUG) && 0
     print_compressed_color_list(tfdev->devn_params.compressed_color_list,
 		    			max(16, num_comp));
 #endif
