@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gp_unix_cache.c,v 1.4 2007/08/01 14:26:00 jemarch Exp $ */
+/* $Id: gp_unix_cache.c,v 1.5 2007/09/11 15:24:39 Arabidopsis Exp $ */
 /* Generic POSIX persistent-cache implementation for Ghostscript */
 
 #include "stdio_.h"
@@ -37,7 +37,7 @@ typedef struct gp_cache_entry_s {
     int type;
     int keylen;
     byte *key;
-    md5_byte_t hash[16];
+    gs_md5_byte_t hash[16];
     char *filename;
     int len;
     void *buffer;
@@ -145,12 +145,12 @@ gp_cache_indexfilename(const char *prefix)
 /* compute and set a cache key's hash */
 private void gp_cache_hash(gp_cache_entry *entry)
 {
-    md5_state_t md5;
+    gs_md5_state_t md5;
     
     /* we use md5 hashes of the key */
-    md5_init(&md5);
-    md5_append(&md5, entry->key, entry->keylen);
-    md5_finish(&md5, entry->hash);
+    gs_md5_init(&md5);
+    gs_md5_append(&md5, entry->key, entry->keylen);
+    gs_md5_finish(&md5, entry->hash);
 }
 
 /* compute and set cache item's filename */
