@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gscdevn.c,v 1.9 2007/08/01 14:26:02 jemarch Exp $ */
+/* $Id: gscdevn.c,v 1.11 2007/09/11 15:24:04 Arabidopsis Exp $ */
 /* DeviceN color space and operation definition */
 
 #include "memory_.h"
@@ -404,10 +404,12 @@ gx_remap_concrete_DeviceN(const frac * pconc, const gs_color_space * pcs,
 	dprintf("gx_remap_concrete_DeviceN: color space id mismatch");
 #endif
 
-#if ENABLE_NAMED_COLOR_CALLBACK
-    if (pis->color_component_map.use_named_color_callback) {
-	return gx_remap_concrete_named_color_DeviceN(pconc, pcs, pdc,
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    if (pis->custom_color_callback) {
+	int code = gx_remap_concrete_custom_color_DeviceN(pconc, pcs, pdc,
 						       	pis, dev, select);
+	if (code >= 0)
+	    return code;
     }
 #endif
     if (pis->color_component_map.use_alt_cspace) {
@@ -505,19 +507,18 @@ check_DeviceN_component_names(const gs_color_space * pcs, gs_state * pgs)
 
 /* Install a DeviceN color space. */
 private int
-gx_install_DeviceN(const gs_color_space * pcs, gs_state * pgs)
+gx_install_DeviceN(gs_color_space * pcs, gs_state * pgs)
 {
     int code;
-#if ENABLE_NAMED_COLOR_CALLBACK
+#if ENABLE_CUSTOM_COLOR_CALLBACK
     /*
-     * Check if we want to use the callback color processing for this color space.
+     * Check if we want to use the callback color processing for this
+     * color space.
      */
-    bool use_named_color_callback =
-    		named_color_callback_install_DeviceN(pgs->color_space, pgs);
+    bool use_custom_color_callback =
+    		custom_color_callback_install_DeviceN(pcs, pgs);
 
-    pgs->color_component_map.use_named_color_callback =
-	   					 use_named_color_callback;
-    if (use_named_color_callback) {
+    if (use_custom_color_callback) {
 	/*
 	 * We are using the callback instead of the alternate tint transform
 	 * for this color space.
@@ -528,6 +529,7 @@ gx_install_DeviceN(const gs_color_space * pcs, gs_state * pgs)
         return 0;
     }
 #endif
+
     code = check_DeviceN_component_names(pcs, pgs);
     if (code < 0)
        return code;
@@ -553,8 +555,15 @@ gx_set_overprint_DeviceN(const gs_color_space * pcs, gs_state * pgs)
 {
     gs_devicen_color_map *  pcmap = &pgs->color_component_map;
 
-    if (pcmap->use_alt_cspace)
-        return gx_spot_colors_set_overprint( pcs->base_space, pgs);
+    if (pcmap->use_alt_cspace) {
+    	const gs_color_space_type* base_type = pcs->base_space->type;
+    	
+    	/* If the base space is DeviceCMYK, handle overprint as DeviceCMYK */
+    	if ( base_type->index == gs_color_space_index_DeviceCMYK )
+    		return base_type->set_overprint( pcs->base_space, pgs );
+    	else
+    		return gx_spot_colors_set_overprint( pcs->base_space, pgs);
+    }
     else {
         gs_overprint_params_t   params;
 
