@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gscie.c,v 1.7 2007/08/01 14:26:02 jemarch Exp $ */
+/* $Id: gscie.c,v 1.9 2007/09/11 15:23:46 Arabidopsis Exp $ */
 /* CIE color rendering cache management */
 #include "math_.h"
 #include "memory_.h"
@@ -421,10 +421,26 @@ gx_install_cie_abc(gs_cie_abc *pcie, gs_state * pgs)
 }
 
 int
-gx_install_CIEDEFG(const gs_color_space * pcs, gs_state * pgs)
+gx_install_CIEDEFG(gs_color_space * pcs, gs_state * pgs)
 {
     gs_cie_defg *pcie = pcs->params.defg;
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        /*
+         * Check if we want to use the callback color processing for this
+         * color space.
+         */
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) pgs->custom_color_callback;
+
+        if (pcb != NULL) {
+	    if (pcb->client_procs->install_CIEBasedDEFG(pcb, pcs, pgs))
+    	        /* Exit if the client will handle the colorspace completely */
+		return 0;
+        }
+    }
+#endif
     CIE_LOAD_CACHE_BODY(pcie->caches_defg.DecodeDEFG, pcie->RangeDEFG.ranges,
 			&pcie->DecodeDEFG, DecodeDEFG_default, pcie,
 			"DecodeDEFG");
@@ -432,10 +448,26 @@ gx_install_CIEDEFG(const gs_color_space * pcs, gs_state * pgs)
 }
 
 int
-gx_install_CIEDEF(const gs_color_space * pcs, gs_state * pgs)
+gx_install_CIEDEF(gs_color_space * pcs, gs_state * pgs)
 {
     gs_cie_def *pcie = pcs->params.def;
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        /*
+         * Check if we want to use the callback color processing for this
+         * color space.
+         */
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) pgs->custom_color_callback;
+
+        if (pcb != NULL) {
+	    if (pcb->client_procs->install_CIEBasedDEF(pcb, pcs, pgs))
+    	        /* Exit if the client will handle the colorspace completely */
+		return 0;
+        }
+    }
+#endif
     CIE_LOAD_CACHE_BODY(pcie->caches_def.DecodeDEF, pcie->RangeDEF.ranges,
 			&pcie->DecodeDEF, DecodeDEF_default, pcie,
 			"DecodeDEF");
@@ -443,18 +475,50 @@ gx_install_CIEDEF(const gs_color_space * pcs, gs_state * pgs)
 }
 
 int
-gx_install_CIEABC(const gs_color_space * pcs, gs_state * pgs)
+gx_install_CIEABC(gs_color_space * pcs, gs_state * pgs)
 {
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        /*
+         * Check if we want to use the callback color processing for this
+         * color space.
+         */
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) pgs->custom_color_callback;
+
+        if (pcb != NULL) {
+	    if (pcb->client_procs->install_CIEBasedABC(pcb, pcs, pgs))
+    	        /* Exit if the client will handle the colorspace completely */
+		return 0;
+        }
+    }
+#endif
     return gx_install_cie_abc(pcs->params.abc, pgs);
 }
 
 int
-gx_install_CIEA(const gs_color_space * pcs, gs_state * pgs)
+gx_install_CIEA(gs_color_space * pcs, gs_state * pgs)
 {
     gs_cie_a *pcie = pcs->params.a;
     gs_sample_loop_params_t lp;
     int i;
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        /*
+         * Check if we want to use the callback color processing for this
+         * color space.
+         */
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) pgs->custom_color_callback;
+
+        if (pcb != NULL) {
+	    if (pcb->client_procs->install_CIEBasedA(pcb, pcs, pgs))
+    	        /* Exit if the client will handle the colorspace completely */
+		return 0;
+        }
+    }
+#endif
     gs_cie_cache_init(&pcie->caches.DecodeA.floats.params, &lp,
 		      &pcie->RangeA, "DecodeA");
     for (i = 0; i <= lp.N; ++i) {
@@ -729,13 +793,13 @@ gs_currentcolorrendering(const gs_state * pgs)
 
 /* Unshare (allocating if necessary) the joint caches. */
 gx_cie_joint_caches *
-gx_currentciecaches(gs_state * pgs)
+gx_unshare_cie_caches(gs_state * pgs)
 {
     gx_cie_joint_caches *pjc = pgs->cie_joint_caches;
 
     rc_unshare_struct(pgs->cie_joint_caches, gx_cie_joint_caches,
 		      &st_joint_caches, pgs->memory,
-		      return 0, "gx_currentciecaches");
+		      return 0, "gx_unshare_cie_caches");
     if (pgs->cie_joint_caches != pjc) {
 	pjc = pgs->cie_joint_caches;
 	pjc->cspace_id = pjc->render_id = gs_no_id;
@@ -744,6 +808,12 @@ gx_currentciecaches(gs_state * pgs)
     return pjc;
 }
 
+gx_cie_joint_caches *
+gx_currentciecaches(gs_state * pgs)
+{
+    return pgs->cie_joint_caches;
+}
+
 /* Compute the parameters for loading a cache, setting base and factor. */
 /* This procedure is idempotent. */
 void
@@ -1147,7 +1217,7 @@ gs_cie_cs_common(const gs_state * pgs)
 int
 gs_cie_cs_complete(gs_state * pgs, bool init)
 {
-    gx_cie_joint_caches *pjc = gx_currentciecaches(pgs);
+    gx_cie_joint_caches *pjc = gx_unshare_cie_caches(pgs);
 
     if (pjc == 0)
 	return_error(gs_error_VMerror);
@@ -1426,6 +1496,7 @@ gx_cie_to_xyz_alloc(gs_imager_state **ppis, const gs_color_space *pcs,
     pjc->skipDecodeABC = pabc != 0 && pabc->caches.skipABC;
     /* Mark the joint caches as completed. */
     pjc->remap_finish = gx_cie_xyz_remap_finish;
+    pjc->cspace_id = pcs->id;
     pjc->status = CIE_JC_STATUS_COMPLETED;
     pis->cie_joint_caches = pjc;
     /*
@@ -1542,3 +1613,24 @@ cie_matrix_init(register gs_matrix3 * mat)
 	mat->cv.v == 1.0 && is_fzero2(mat->cv.u, mat->cv.w) &&
 	mat->cw.w == 1.0 && is_fzero2(mat->cw.u, mat->cw.v);
 }
+
+bool 
+gx_color_space_needs_cie_caches(const gs_color_space * pcs)
+{
+    switch (pcs->type->index) {
+    	case gs_color_space_index_CIEDEFG:
+    	case gs_color_space_index_CIEDEF:
+    	case gs_color_space_index_CIEABC:
+    	case gs_color_space_index_CIEA:
+    	case gs_color_space_index_CIEICC:
+	    return true;
+	case gs_color_space_index_DevicePixel:
+    	case gs_color_space_index_DeviceN:
+    	case gs_color_space_index_Separation:
+    	case gs_color_space_index_Indexed:
+    	case gs_color_space_index_Pattern:
+	    return gx_color_space_needs_cie_caches(pcs->base_space);
+	default:
+	    return false;
+    }
+}
