@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gscolor3.c,v 1.8 2007/08/01 14:26:03 jemarch Exp $ */
+/* $Id: gscolor3.c,v 1.9 2007/09/11 15:23:59 Arabidopsis Exp $ */
 /* "Operators" for LanguageLevel 3 color facilities */
 #include "gx.h"
 #include "gserrors.h"
@@ -32,6 +32,7 @@
 #include "gzpath.h"
 #include "gxpaint.h"		/* (requires gx_path) */
 #include "gxshade.h"
+#include "gscie.h"
 
 /* setsmoothness */
 int
@@ -57,19 +58,30 @@ gs_shfill(gs_state * pgs, const gs_shading_t * psh)
      * shfill is equivalent to filling the current clipping path (or, if
      * clipping, its bounding box) with the shading, disregarding the
      * Background if any.  In order to produce reasonable high-level output,
-     * we must actually implement this by calling gs_fill rather than
-     * gs_shading_fill_path.  However, filling with a shading pattern does
-     * paint the Background, so if necessary, we construct a copy of the
-     * shading with Background removed.
+     * we must implement this by calling gs_fill_path. 
      */
     gs_pattern2_template_t pat;
-    gx_path cpath;
     gs_matrix imat;
     gs_client_color cc;
     gs_color_space *pcs;
     gx_device_color devc;
     int code;
 
+    /* Must install the shading color space
+       to allow check_DeviceN_component_names initialize
+       the color component map.
+     */
+    /* Don't bother with saving the old color space, color,
+       and cie_joint_caches,
+       because .shfill is always called within gsave-grestore -
+       see gs/lib . */
+    code = gs_setcolorspace(pgs, psh->params.ColorSpace);
+    if (code < 0)
+	return 0;
+    if (psh->params.cie_joint_caches != NULL) {
+	pgs->cie_joint_caches = psh->params.cie_joint_caches;
+	rc_increment(pgs->cie_joint_caches);
+    }
     gs_pattern2_init(&pat);
     pat.Shading = psh;
     gs_make_identity(&imat);
@@ -88,12 +100,22 @@ gs_shfill(gs_state * pgs, const gs_shading_t * psh)
     code = pcs->type->remap_color(&cc, pcs, &devc, (gs_imager_state *)pgs,
 				  pgs->device, gs_color_select_texture);
     if (code >= 0) {
-	gx_path_init_local(&cpath, pgs->memory);
-	code = gx_cpath_to_path(pgs->clip_path, &cpath);
-	if (code >= 0)
-	    code = gx_fill_path(&cpath, &devc, pgs, gx_rule_winding_number,
+	gx_device *dev = pgs->device;
+	bool need_path = !dev_proc(dev, pattern_manage)(dev, gs_no_id, NULL,
+		pattern_manage__shfill_doesnt_need_path);
+
+	if (need_path) {
+    	    gx_path cpath;
+
+	    gx_path_init_local(&cpath, pgs->memory);
+	    code = gx_cpath_to_path(pgs->clip_path, &cpath);
+	    if (code >= 0)
+		code = gx_fill_path(&cpath, &devc, pgs, gx_rule_winding_number,
+				    pgs->fill_adjust.x, pgs->fill_adjust.y);
+	    gx_path_free(&cpath, "gs_shfill");
+	} else 
+	    code = gx_fill_path(NULL, &devc, pgs, gx_rule_winding_number,
 				pgs->fill_adjust.x, pgs->fill_adjust.y);
-	gx_path_free(&cpath, "gs_shfill");
     }
     rc_decrement(pcs, "gs_shfill");
     gs_pattern_reference(&cc, -1);
