@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gscscie.c,v 1.8 2007/08/01 14:26:03 jemarch Exp $ */
+/* $Id: gscscie.c,v 1.10 2007/09/11 15:24:13 Arabidopsis Exp $ */
 /* CIE color space management */
 #include "math_.h"
 #include "gx.h"
@@ -57,7 +57,12 @@ const gs_color_space_type gs_color_space_type_CIEDEFG = {
     gx_init_CIE, gx_restrict_CIEDEFG,
     gx_concrete_space_CIE,
     gx_concretize_CIEDEFG, NULL,
-    gx_default_remap_color, gx_install_CIE,
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    gx_remap_CIEDEFG,
+#else
+    gx_default_remap_color,
+#endif
+    gx_install_CIE,
     gx_spot_colors_set_overprint,
     gx_final_CIEDEFG, gx_no_adjust_color_count,
     gx_serialize_CIEDEFG,
@@ -76,7 +81,12 @@ const gs_color_space_type gs_color_space_type_CIEDEF = {
     gx_init_CIE, gx_restrict_CIEDEF,
     gx_concrete_space_CIE,
     gx_concretize_CIEDEF, NULL,
-    gx_default_remap_color, gx_install_CIE,
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    gx_remap_CIEDEF,
+#else
+    gx_default_remap_color,
+#endif
+    gx_install_CIE,
     gx_spot_colors_set_overprint,
     gx_final_CIEDEF, gx_no_adjust_color_count,
     gx_serialize_CIEDEF,
@@ -114,7 +124,12 @@ const gs_color_space_type gs_color_space_type_CIEA = {
     gx_init_CIE, gx_restrict_CIEA,
     gx_concrete_space_CIE,
     gx_concretize_CIEA, NULL,
-    gx_default_remap_color, gx_install_CIE,
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    gx_remap_CIEA,
+#else
+    gx_default_remap_color,
+#endif
+    gx_install_CIE,
     gx_spot_colors_set_overprint,
     gx_final_CIEA, gx_no_adjust_color_count,
     gx_serialize_CIEA,
@@ -141,7 +156,7 @@ gx_concrete_space_CIE(const gs_color_space * pcs, const gs_imager_state * pis)
 /* interpreters can substitute their own installer. */
 /* This procedure is exported for the benefit of gsicc.c */
 int
-gx_install_CIE(const gs_color_space * pcs, gs_state * pgs)
+gx_install_CIE(gs_color_space * pcs, gs_state * pgs)
 {
     return (*pcs->params.a->common.install_cspace) (pcs, pgs);
 }
