@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gscspace.h,v 1.8 2007/08/01 14:26:04 jemarch Exp $ */
+/* $Id: gscspace.h,v 1.10 2007/09/11 15:24:40 Arabidopsis Exp $ */
 /* Client interface to color spaces */
 
 #ifndef gscspace_INCLUDED
@@ -25,7 +25,6 @@
 
 #include "gsmemory.h"
 #include "gsiparam.h"
-#include "gsnamecl.h"
 #include "gsrefct.h"
 
 /*
@@ -194,9 +193,6 @@ typedef struct gs_separation_params_s {
     separation_type sep_type;
     bool use_alt_cspace;
     gs_callback_func_get_colorname_string *get_colorname_string;
-#if ENABLE_NAMED_COLOR_CALLBACK
-    named_color_params_t named_color_params;
-#endif
 } gs_separation_params;
 
 typedef struct gs_device_n_params_s {
@@ -206,11 +202,11 @@ typedef struct gs_device_n_params_s {
     gs_device_n_attributes *colorants;
     bool use_alt_cspace;
     gs_callback_func_get_colorname_string *get_colorname_string;
-#if ENABLE_NAMED_COLOR_CALLBACK
-    named_color_params_t named_color_params;
-#endif
 } gs_device_n_params;
 
+/* Define an abstract type for the client color space data */
+typedef struct client_color_space_data_s client_color_space_data_t;
+
 /*
  * Non-direct paint space: Indexed space.
  *
@@ -257,6 +253,7 @@ struct gs_color_space_s {
     rc_header                  rc;
     gs_id                      id;
     gs_color_space             *base_space;
+    client_color_space_data_t	*pclient_color_space_data;
     union {
 	gs_device_pixel_params   pixel;
 	gs_cie_defg *            defg;
