@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gsdevice.c,v 1.9 2007/08/01 14:26:04 jemarch Exp $ */
+/* $Id: gsdevice.c,v 1.11 2007/09/11 15:24:38 Arabidopsis Exp $ */
 /* Device operators for Ghostscript library */
 #include "ctype_.h"
 #include "memory_.h"		/* for memchr, memcpy */
@@ -297,8 +297,11 @@ gs_copydevice2(gx_device ** pnew_dev, const gx_device * dev, bool keep_open,
     code = dev_proc(new_dev, finish_copydevice)(new_dev, dev);
     if (code < 0) {
 	gs_free_object(mem, new_dev, "gs_copydevice(device)");
+#if 0 /* gs_free_object above calls gx_device_finalize,
+	 which closes the device and releaszes its stype, i.e. a_std. */
 	if (a_std)
 	    gs_free_object(dev->memory->non_gc_memory, a_std, "gs_copydevice(stype)");
+#endif
 	return code;
     }
     *pnew_dev = new_dev;
