@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gsdparam.c,v 1.9 2007/08/01 14:26:04 jemarch Exp $ */
+/* $Id: gsdparam.c,v 1.11 2007/09/11 15:24:14 Arabidopsis Exp $ */
 /* Default device parameters for Ghostscript library */
 #include "memory_.h"		/* for memcpy */
 #include "string_.h"		/* for strlen */
@@ -123,12 +123,6 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
 
     /* Transmit the values. */
 
-#if ENABLE_NAMED_COLOR_CALLBACK
-    /* The named color callback pointer */
-    code = named_color_callback_get_params(dev, plist);
-    if (code < 0)
-	return code;
-#endif
     if (
 
 	/* Standard parameters */
@@ -394,6 +388,7 @@ gdev_end_output_media(gs_param_list * mlist, gs_param_dict * pdict)
 /* ================ Putting parameters ================ */
 
 /* Forward references */
+private int param_normalize_anti_alias_bits( uint max_gray, int bits );
 private int param_anti_alias_bits(gs_param_list *, gs_param_name, int *);
 private int param_MediaSize(gs_param_list *, gs_param_name,
 			    const float *, gs_param_float_array *);
@@ -487,12 +482,6 @@ e:	param_signal_error(plist, param_name, ecode);\
     }\
     END
 
-#if ENABLE_NAMED_COLOR_CALLBACK
-    /* The named_color callback pointer */
-    code = named_color_callback_put_params(dev, plist);
-    if (code < 0)
-	ecode = code;
-#endif
     /*
      * The actual value of LeadingEdge must be changed inside this routine,
      * so that we can detect that it has been changed. Thus, instead of a
@@ -820,8 +809,12 @@ nce:
 	dev->ImagingBBox_set = false;
     }
     dev->UseCIEColor = ucc;
-    dev->color_info.anti_alias.text_bits = tab;
-    dev->color_info.anti_alias.graphics_bits = gab;
+	dev->color_info.anti_alias.text_bits =
+		param_normalize_anti_alias_bits(max(dev->color_info.max_gray,
+			dev->color_info.max_color), tab);
+	dev->color_info.anti_alias.graphics_bits =
+		param_normalize_anti_alias_bits(max(dev->color_info.max_gray,
+			dev->color_info.max_color), gab);
     dev->LockSafetyParams = locksafe;
     gx_device_decache_colors(dev);
     return 0;
@@ -835,6 +828,17 @@ gx_device_request_leadingedge(gx_device *dev, int le_req)
 	LEADINGEDGE_REQ_BIT;
 }
 
+/* Limit the anti-alias bit values to the maximum legal value for the
+ * current color depth.
+ */
+private int
+param_normalize_anti_alias_bits( uint max_gray, int bits )
+{
+	int	max_bits = ilog2( max_gray + 1);
+	
+	return  (bits > max_bits ? max_bits : bits);
+}
+
 /* Read TextAlphaBits or GraphicsAlphaBits. */
 private int
 param_anti_alias_bits(gs_param_list * plist, gs_param_name param_name, int *pa)
