@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -16,7 +16,7 @@
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 */
-/* $Id: gsnamecl.c,v 1.3 2007/08/01 14:26:09 jemarch Exp $ */
+/* $Id: gsnamecl.c,v 1.5 2007/09/11 15:24:02 Arabidopsis Exp $ */
 /* Callback mechanism for handling color spaces containing named colors. */
 
 /*
@@ -45,19 +45,19 @@
 #include "gscspace.h"
 #include "gxdevice.h"
 #include "gzstate.h"
+#include "gsutil.h"
 
-#if ENABLE_NAMED_COLOR_CALLBACK		/* Defined in src/gsnamecl.h */
-
+#if ENABLE_CUSTOM_COLOR_CALLBACK	/* Defined in src/gsnamecl.h */
+  
 /*
  * Check if we want to use the callback color processing logic for the given
  * Separation color space.
  */
 bool
-named_color_callback_install_Separation(gs_color_space * pcs, gs_state * pgs)
+custom_color_callback_install_Separation(gs_color_space * pcs, gs_state * pgs)
 {
-    gx_device * dev = pgs->device;
-    client_named_color_params_t * pcb =
-	(client_named_color_params_t *) dev->named_color_callback;
+	client_custom_color_params_t * pcb =
+	(client_custom_color_params_t *) pgs->custom_color_callback;
 
     return (pcb == NULL) ? false
 	    		 : pcb->client_procs->install_Separation(pcb, pcs, pgs);
@@ -68,30 +68,29 @@ named_color_callback_install_Separation(gs_color_space * pcs, gs_state * pgs)
  * DeviceN color space.
  */
 bool
-named_color_callback_install_DeviceN(gs_color_space * pcs, gs_state * pgs)
+custom_color_callback_install_DeviceN(gs_color_space * pcs, gs_state * pgs)
 {
-    gx_device * dev = pgs->device;
-    client_named_color_params_t * pcb =
-	(client_named_color_params_t *) dev->named_color_callback;
-
+    client_custom_color_params_t * pcb =
+	(client_custom_color_params_t *) pgs->custom_color_callback;
+  
     return (pcb == NULL) ? false
-	    		 : pcb->client_procs->install_DeviceN(pcb, pcs, pgs);
-}
-
+  	    		 : pcb->client_procs->install_DeviceN(pcb, pcs, pgs);
+  }
+  
 /*
- * Convert a Separation color using the 'named color' callback into device color.
- */
+ * Convert a Separation color using the 'custom color' callback into
+ * device color.
+*/
 int
-gx_remap_concrete_named_color_Separation(const frac * pconc,
-	const gs_color_space * pcs, gx_device_color * pdc,
-       	const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
+gx_remap_concrete_custom_color_Separation(const frac * pconc,
+  	const gs_color_space * pcs, gx_device_color * pdc,
+         	const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
 {
-    client_named_color_params_t * pcb =
-	(client_named_color_params_t *) dev->named_color_callback;
-
-    if (pcb == NULL) {
-	dlprintf("ERROR:  Named color callback is NULL\n");
-	return_error(gs_error_rangecheck);
+    client_custom_color_params_t * pcb =
+	(client_custom_color_params_t *) pis->custom_color_callback;
+  
+      if (pcb == NULL) {
+  	return_error(gs_error_rangecheck);
     }
     else
 	return pcb->client_procs->remap_Separation(pcb, pconc,
@@ -99,119 +98,124 @@ gx_remap_concrete_named_color_Separation(const frac * pconc,
 }
 
 /*
- * Convert a DeviceN color using the 'named color' callback into device color.
+ * Convert a DeviceN color using the 'custom color' callback into device
+ * color.
  */
 int
-gx_remap_concrete_named_color_DeviceN(const frac * pconc,
+gx_remap_concrete_custom_color_DeviceN(const frac * pconc,
 	const gs_color_space * pcs, gx_device_color * pdc,
        	const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
 {
-    client_named_color_params_t * pcb =
-	(client_named_color_params_t *) dev->named_color_callback;
-
-    if (pcb == NULL) {
-	dlprintf("ERROR:  Named color callback is NULL\n");
-	return_error(gs_error_rangecheck);
+    client_custom_color_params_t * pcb =
+	(client_custom_color_params_t *) pis->custom_color_callback;
+  
+	if (pcb == NULL) {
+  		return_error(gs_error_rangecheck);
     }
     else
-	return pcb->client_procs->remap_DeviceN(pcb, pconc,
+		return pcb->client_procs->remap_DeviceN(pcb, pconc,
 					pcs, pdc, pis, dev, select);
 }
 
 /*
- * Get the 'Named color' client callback parameter block pointer.  This value
+ * Get the 'custom color' client callback parameter block pointer.  This value
  * is passed as a string type device paramter.  A string is being used since
  * PostScript does not support pointers as a type.  Note:  An integer type
  * is not being used since PS intergers are nominally 32 bits.  Thus there
  * would be a problem using integers to pass pointer values on 64 bit systems.
  */
 int
-named_color_callback_get_params(gx_device * dev, gs_param_list * plist)
+custom_color_callback_get_params(gs_state * pgs, gs_param_list * plist)
 {
     /* Convert our pointer to a PostScript hex string */
     char buf[64] = "16#";
     int buf_pos = 3;
-    gs_param_string named_color_param;
+    gs_param_string custom_color_param;
     int idx;
     int val;
     size_t iptr;
-   
+ 
     idx = ((int)sizeof(size_t)) * 8 - 4;
-    iptr = (size_t)(dev->named_color_callback);
+    iptr = (size_t)(pgs->custom_color_callback);
     while (idx >= 0) {
 	val = (int)(iptr >> idx) & 0xf;
-        if (val <= 9)
-	    buf[buf_pos++] = '0' + val;
+    if (val <= 9)
+		buf[buf_pos++] = '0' + val;
 	else
 	    buf[buf_pos++] = 'a' - 10 + val;
 	idx -= 4;
     }
-    buf[buf_pos] = '\0';
-     
-    param_string_from_transient_string(named_color_param, buf);
+    param_string_from_transient_string(custom_color_param, buf);
 
-    return param_write_string(plist, "NamedColorCallback", &named_color_param);
+    return param_write_string(plist, CustomColorCallbackParamName,
+		   			 &custom_color_param);
 }
 
 /*
- * Put the 'Named color' client callback parameter block pointer.  This value
+ * Put the 'custom color' client callback parameter block pointer.  This value
  * is passed as a string type device paramter.  A string is being used since
  * PostScript does not support pointers as a type.  Note:  An integer type
  * is not being used since PS integers are nominally 32 bits.  Thus there
  * would be a problem using integers to pass pointer values on 64 bit systems.
  */
 int
-named_color_callback_put_params(gx_device * dev, gs_param_list * plist)
+custom_color_callback_put_params(gs_state * pgs, gs_param_list * plist)
 {
     int code;
-    size_t iptr = (size_t)(dev->named_color_callback);
+    size_t iptr = (size_t)(pgs->custom_color_callback);
     gs_param_string dh = { 0 };
 
-    switch (code = param_read_string(plist, "NamedColorCallback", &dh)) {
+    switch (code = param_read_string(plist, CustomColorCallbackParamName, &dh)) {
 	case 0:
-	    {
-	        /* 
-	         * Convert from a string to a pointer.  
-	         * It is assumed that size_t has the same size as a pointer.
-	         * Allow formats (1234), (10#1234) or (16#04d2).
-	         */
- 	        int i;
-	        int base = 10;
- 	        int val;
-	        code = 0;
-	        for (i = 0; i < dh.size; i++) {
-	            val = dh.data[i];
-	            if ((val >= '0') && (val <= '9'))
-		        val = val - '0';
-	            else if ((val >= 'A') && (val <= 'F'))
+	{
+       /* 
+        * Convert from a string to a pointer.  
+        * It is assumed that size_t has the same size as a pointer.
+        * Allow formats (1234), (10#1234) or (16#04d2).
+        */
+		uint i;
+		int base = 10;
+		int val;
+		code = 0;
+		for (i = 0; i < dh.size; i++) {
+			val = dh.data[i];
+			if ((val >= '0') && (val <= '9'))
+				val = val - '0';
+			else if ((val >= 'A') && (val <= 'F'))
 		        val = val - 'A' + 10;
-	            else if ((val >= 'a') && (val <= 'f'))
+			else if ((val >= 'a') && (val <= 'f'))
 		        val = val - 'a' + 10;
-	            else if (val == '#' && ((iptr == 10) || (iptr == 16))) {
+			else if (val == '#' && ((iptr == 10) || (iptr == 16))) {
 		        base = (int) iptr;
 		        iptr = 0;
 		        continue;
-	            }
-	            else {
+			}
+			else {
 		        code = gs_error_rangecheck;
 		        break;
-	            }
-	            iptr = iptr * base + val;
-	        }
-	    }
-	    break;
+			}
+			iptr = iptr * base + val;
+		}
+	}
+	break;
 	default:
 	case 1:
 	    dh.data = 0;
 	    break;
     }
     if (code < 0) {
-	param_signal_error(plist, "NamedColorCallback", code);
+	param_signal_error(plist, "CustomColorCallback", code);
+    }
+    else if (pgs->custom_color_callback != (void *)iptr) {
+        pgs->custom_color_callback = (void *)iptr;
+	/*
+	 * Custom color processing can depend upon the type of object
+	 * being imaged so we enable object type tagging.
+	 */
+    gs_enable_object_tagging();
     }
-    else
-        dev->named_color_callback = (void *)iptr;
 
     return 0;
 }
 
-#endif 			/* ENABLE_NAMED_COLOR_CALLBACK */
+#endif 			/* ENABLE_CUSTOM_COLOR_CALLBACK */
