@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gsptype1.c,v 1.9 2007/08/01 14:26:12 jemarch Exp $ */
+/* $Id: gsptype1.c,v 1.11 2007/09/11 15:24:35 Arabidopsis Exp $ */
 /* PatternType 1 pattern implementation */
 #include "math_.h"
 #include "gx.h"
@@ -455,6 +455,13 @@ gs_pattern1_get_pattern(const gs_pattern_instance_t *pinst)
 	&((const gs_pattern1_instance_t *)pinst)->template;
 }
 
+/* Check device color for Pattern Type 1. */
+bool
+gx_dc_is_pattern1_color(const gx_device_color *pdevc)
+{
+    return pdevc->type == &gx_dc_pattern;
+}
+
 /*
  * Perform actions required at setcolor time. This procedure resets the
  * overprint information (almost) as required by the pattern. The logic
