@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gsptype2.c,v 1.9 2007/08/01 14:26:12 jemarch Exp $ */
+/* $Id: gsptype2.c,v 1.10 2007/09/11 15:23:47 Arabidopsis Exp $ */
 /* PatternType 2 implementation */
 #include "gx.h"
 #include "gserrors.h"
@@ -210,20 +210,11 @@ gs_pattern2_set_color(const gs_client_color * pcc, gs_state * pgs)
     return code;
 }
 
-/* Fill path or rect, and with a PatternType 2 color. */
-int
-gx_dc_pattern2_fill_path(const gx_device_color * pdevc, 
-                              gx_path * ppath, gs_fixed_rect * rect, 
-                              gx_device * dev)
-{
-    gs_pattern2_instance_t *pinst =
-        (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
-
-    return gs_shading_fill_path_adjusted(pinst->template.Shading, ppath, rect, dev,
-                                (gs_imager_state *)pinst->saved, !pinst->shfill);
-}
-
 /* Fill a rectangle with a PatternType 2 color. */
+/* WARNING: This function doesn't account the shading BBox
+   to allow the clipent to optimize the clipping 
+   with changing the order of clip patrhs and rects.
+   The client must clip with the shading BBOx before calling this function. */
 private int
 gx_dc_pattern2_fill_rectangle(const gx_device_color * pdevc, int x, int y,
                               int w, int h, gx_device * dev,
@@ -236,12 +227,15 @@ gx_dc_pattern2_fill_rectangle(const gx_device_color * pdevc, int x, int y,
 	return dev_proc(dev, fill_rectangle)(dev, x, y, w, h, (gx_color_index)0/*any*/);
     } else {
 	gs_fixed_rect rect;
+	gs_pattern2_instance_t *pinst =
+	    (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
 
 	rect.p.x = int2fixed(x);
 	rect.p.y = int2fixed(y);
 	rect.q.x = int2fixed(x + w);
 	rect.q.y = int2fixed(y + h);
-	return gx_dc_pattern2_fill_path(pdevc, NULL, &rect,  dev);
+	return gs_shading_do_fill_rectangle(pinst->template.Shading, &rect, dev,
+				    (gs_imager_state *)pinst->saved, !pinst->shfill);
     }
 }
 
@@ -309,6 +303,59 @@ gx_dc_pattern2_get_bbox(const gx_device_color * pdevc, gs_fixed_rect *bbox)
     return 1;
 }
 
+int
+gx_dc_pattern2_color_has_bbox(const gx_device_color * pdevc)
+{
+    gs_pattern2_instance_t *pinst = (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
+    const gs_shading_t *psh = pinst->template.Shading;
+
+    return psh->params.have_BBox;
+}
+
+/* Create a path from a PatternType 2 shading BBox to a path. */
+private int
+gx_dc_shading_path_add_box(gx_path *ppath, const gx_device_color * pdevc)
+{
+    gs_pattern2_instance_t *pinst = (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
+    const gs_shading_t *psh = pinst->template.Shading;
+
+    if (!psh->params.have_BBox)
+	return_error(gs_error_unregistered); /* Do not call in this case. */
+    else {
+	gs_state *pis = pinst->saved;
+
+	return gs_shading_path_add_box(ppath, &psh->params.BBox, &pis->ctm);
+    }
+}
+
+/* Intersect a clipping path a shading BBox. */
+int
+gx_dc_pattern2_clip_with_bbox(const gx_device_color * pdevc, gx_device * pdev, 
+			      gx_clip_path *cpath_local, const gx_clip_path **ppcpath1)
+{
+    if (gx_dc_is_pattern2_color(pdevc) && gx_dc_pattern2_color_has_bbox(pdevc) &&
+	    (*dev_proc(pdev, pattern_manage))(pdev, gs_no_id, NULL, pattern_manage__shading_area) == 0) {
+	gs_pattern2_instance_t *pinst = (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
+	gx_path box_path;
+	int code;
+
+	gx_path_init_local(&box_path, pdev->memory);
+	code = gx_dc_shading_path_add_box(&box_path, pdevc);
+	if (code == gs_error_limitcheck) {
+	    /* Ignore huge BBox - bug 689027. */
+	    code = 0;
+	} else {
+	    if (code >= 0) {
+		gx_cpath_init_local_shared(cpath_local, *ppcpath1, pdev->memory);
+		code = gx_cpath_intersect(cpath_local, &box_path, gx_rule_winding_number, (gs_imager_state *)pinst->saved);
+		*ppcpath1 = cpath_local;
+	    }
+	}
+	gx_path_free(&box_path, "gx_default_fill_path(path_bbox)");
+    }
+    return 0;
+}
+
 /* Get a shading color space. */
 const gs_color_space *
 gx_dc_pattern2_get_color_space(const gx_device_color * pdevc)
