@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gsstate.c,v 1.8 2007/08/01 14:26:14 jemarch Exp $ */
+/* $Id: gsstate.c,v 1.10 2007/09/11 15:24:08 Arabidopsis Exp $ */
 /* Miscellaneous graphics state operators for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -705,6 +705,29 @@ gs_currentoverprintmode(const gs_state * pgs)
     return pgs->overprint_mode;
 }
 
+/* setrenderingintent
+ *
+ *  Use ICC numbers from Table 18 (section 6.1.11) rather than the PDF order
+ *  to reduce re-coding and confusion.
+ *    Perceptual            0
+ *    Relative Colorimetric 1
+ *    Saturation            2
+ *    AbsoluteColorimetric  3
+ */
+int
+gs_setrenderingintent(gs_state *pgs, int ri) {
+    if (ri < 0 || ri > 3)
+	return_error(gs_error_rangecheck);
+    pgs->renderingintent = ri;
+    return 0;
+}
+
+/* currentrenderingintent */
+int
+gs_currentrenderingintent(const gs_state * pgs)
+{
+    return pgs->renderingintent;
+}
 
 /*
  * Reset most of the graphics state.
