@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gstext.c,v 1.9 2007/08/01 14:26:14 jemarch Exp $ */
+/* $Id: gstext.c,v 1.11 2007/09/11 15:24:35 Arabidopsis Exp $ */
 /* Driver text interface support */
 
 #include "memory_.h"
@@ -262,6 +262,9 @@ gs_text_begin(gs_state * pgs, const gs_text_params_t * text,
        of a Type 3 font while stringwidth. 
        Unfortunately we can't effectively know a leaf font type here,
        so we load the color unconditionally . */
+    /* Processing a text object operation */
+    gs_set_object_tag(pgs, GS_TEXT_TAG);
+
     gx_set_dev_color(pgs);
     code = gs_state_color_load(pgs);
     if (code < 0)
@@ -286,6 +289,9 @@ gs_text_update_dev_color(gs_state * pgs, gs_text_enum_t * pte)
      * update of the graphic state color will update the text color as
      * well.
      */
+    /* Processing a text object operation */
+    gs_set_object_tag(pgs, GS_TEXT_TAG);
+
     if (pte->pdcolor != 0)
         gx_set_dev_color(pgs);
     return 0;
