@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gstype42.c,v 1.9 2007/08/01 14:26:15 jemarch Exp $ */
+/* $Id: gstype42.c,v 1.11 2007/09/11 15:23:49 Arabidopsis Exp $ */
 /* Type 42 (TrueType) font library routines */
 #include "memory_.h"
 #include "gx.h"
@@ -40,6 +40,7 @@
 #include "gxistate.h"
 #include "gzstate.h"
 #include "stream.h"
+#include <stdlib.h>		/* for qsort */
 
 /* Structure descriptor */
 public_st_gs_font_type42();
@@ -54,15 +55,6 @@ private int default_get_outline(gs_font_type42 *pfont, uint glyph_index,
 font_proc_font_info(gs_type42_font_info); /* Type check. */
 font_proc_font_info(gs_truetype_font_info); /* Type check. */
 
-/* Set up a pointer to a substring of the font data. */
-/* Free variables: pfont, string_proc. */
-#define ACCESS(base, length, vptr)\
-  BEGIN\
-    code = (*string_proc)(pfont, (ulong)(base), length, &vptr);\
-    if ( code < 0 ) return code;\
-    if ( code > 0 ) return_error(gs_error_invalidfont);\
-  END
-
 /* Get 2- or 4-byte quantities from a table. */
 #define U8(p) ((uint)((p)[0]))
 #define S8(p) (int)((U8(p) ^ 0x80) - 0x80)
@@ -74,44 +66,89 @@ font_proc_font_info(gs_truetype_font_info); /* Type check. */
 
 GS_NOTIFY_PROC(gs_len_glyphs_release);
 
+/* Read data from sfnts. */
+int
+gs_type42_read_data(gs_font_type42 * pfont, ulong pos, uint length, byte *buf)
+{
+    int (*string_proc)(gs_font_type42 *, ulong, uint, const byte **) =
+	pfont->data.string_proc;
+    uint left = length;
+    const byte *data;
+    int code;
+
+    do {
+	code = (*string_proc)(pfont, (ulong)(pos + length - left), left, &data);
+	if (code < 0) 
+	    return code;
+	if (code == 0) 
+	    code = left;
+	memcpy(buf + length - left, data, code);
+	left -= code;
+    } while (left);
+    return 0;
+}
+
 /* Get the offset to a glyph using the loca table */
 private inline ulong
 get_glyph_offset(gs_font_type42 *pfont, uint glyph_index) 
 {
-    int (*string_proc)(gs_font_type42 *, ulong, uint, const byte **) =
-	pfont->data.string_proc;
-    const byte *ploca;
     ulong result;
-    int code;		/* hidden variable used by ACCESS */
+    byte buf[4];
+    int code;
 
+    /* Hack : when reading fails, we return a huge offset. 
+       Then gs_type42_font_init will fails because it is greater than loca size. 
+       Fixme : improve with changing the function prototype.
+     */
     if (pfont->data.indexToLocFormat) {
-	ACCESS(pfont->data.loca + glyph_index * 4, 4, ploca);
-	result = u32(ploca);
+	code = gs_type42_read_data(pfont, pfont->data.loca + glyph_index * 4, 4, buf);
+	result = (code < 0 ? 0xffffffff : u32(buf));
     } else {
-	ACCESS(pfont->data.loca + glyph_index * 2, 2, ploca);
-	result = (ulong) U16(ploca) << 1;
+	code = gs_type42_read_data(pfont, pfont->data.loca + glyph_index * 2, 2, buf);
+	result = (code < 0 ? 0xffffffff : (ulong) U16(buf) << 1);
     }
     return result;
 }
 
+/* compare fn used by gs_type42_font_init() for sorting the 'loca' table */
+typedef struct gs_type42_font_init_sort_s {
+    ulong glyph_offset;
+    int glyph_num;
+} gs_type42_font_init_sort_t;
+private int
+gs_type42_font_init_compare (const void *a, const void *b)
+{
+    ulong a_offset = ((const gs_type42_font_init_sort_t *)a)->glyph_offset;
+    ulong b_offset = ((const gs_type42_font_init_sort_t *)b)->glyph_offset;
+    if (a_offset < b_offset)
+	return -1;
+    else if (a_offset > b_offset)
+	return +1;
+    else
+	/* make the sort stable */
+	return ((const gs_type42_font_init_sort_t *)a)->glyph_num -
+	       ((const gs_type42_font_init_sort_t *)b)->glyph_num;
+}
+
 /*
  * Initialize the cached values in a Type 42 font.
  * Note that this initializes the type42_data procedures other than
  * string_proc, and the font procedures as well.
  */
+
 int
 gs_type42_font_init(gs_font_type42 * pfont, int subfontID)
 {
     int (*string_proc)(gs_font_type42 *, ulong, uint, const byte **) =
 	pfont->data.string_proc;
-    const byte *OffsetTable;
+    byte OffsetTable[12];
     uint numTables;
-    const byte *TableDirectory;
+    byte TableDirectory[MAX_NUM_TT_TABLES * 16];
     uint i;
     int code;
     byte head_box[8];
     ulong loca_size = 0;
-    ulong glyph_start, glyph_offset, glyph_length;
+    ulong glyph_start, glyph_offset, glyph_length, glyph_size = 0;
     uint numFonts;
     uint OffsetTableOffset;
 
@@ -119,15 +156,15 @@ gs_type42_font_init(gs_font_type42 * pfont, int subfontID)
     static const byte version_true[4] = {'t', 'r', 'u', 'e'};
     static const byte version_ttcf[4] = {'t', 't', 'c', 'f'};
 
-    ACCESS(0, 12, OffsetTable);
+    READ_SFNTS(pfont, 0, 12, OffsetTable);
     if (!memcmp(OffsetTable, version_ttcf, 4))
     {
 	numFonts = u32(OffsetTable + 8);
 	if (subfontID < 0 || subfontID >= numFonts)
 	    return_error(gs_error_rangecheck);
-	ACCESS(12, numFonts * 4, OffsetTable);
-	OffsetTableOffset = u32(OffsetTable + subfontID * 4);
-	ACCESS(OffsetTableOffset, 12, OffsetTable);
+	READ_SFNTS(pfont, 12 + subfontID * 4, 4, OffsetTable);
+	OffsetTableOffset = u32(OffsetTable);
+	READ_SFNTS(pfont, OffsetTableOffset, 12, OffsetTable);
     }
     else
     {
@@ -139,7 +176,9 @@ gs_type42_font_init(gs_font_type42 * pfont, int subfontID)
 	return_error(gs_error_invalidfont);
 
     numTables = U16(OffsetTable + 4);
-    ACCESS(OffsetTableOffset + 12, numTables * 16, TableDirectory);
+    if (numTables > MAX_NUM_TT_TABLES)
+	return_error(gs_error_invalidfont);
+    READ_SFNTS(pfont, OffsetTableOffset + 12, numTables * 16, TableDirectory);
     /* Clear all non-client-supplied data. */
     {
 	void *proc_data = pfont->data.proc_data;
@@ -154,19 +193,20 @@ gs_type42_font_init(gs_font_type42 * pfont, int subfontID)
 
 	if (!memcmp(tab, "cmap", 4))
 	    pfont->data.cmap = offset;
-	else if (!memcmp(tab, "glyf", 4))
+	else if (!memcmp(tab, "glyf", 4)) {
 	    pfont->data.glyf = offset;
-	else if (!memcmp(tab, "head", 4)) {
-	    const byte *head;
+	    glyph_size = (uint)u32(tab + 12);
+	} else if (!memcmp(tab, "head", 4)) {
+	    byte head[54];
 
-	    ACCESS(offset, 54, head);
+	    READ_SFNTS(pfont, offset, 54, head);
 	    pfont->data.unitsPerEm = U16(head + 18);
 	    memcpy(head_box, head + 36, 8);
 	    pfont->data.indexToLocFormat = U16(head + 50);
 	} else if (!memcmp(tab, "hhea", 4)) {
-	    const byte *hhea;
+	    byte hhea[36];
 
-	    ACCESS(offset, 36, hhea);
+	    READ_SFNTS(pfont, offset, 36, hhea);
 	    pfont->data.metrics[0].numMetrics = U16(hhea + 34);
 	} else if (!memcmp(tab, "hmtx", 4)) {
 	    pfont->data.metrics[0].offset = offset;
@@ -175,16 +215,16 @@ gs_type42_font_init(gs_font_type42 * pfont, int subfontID)
 	    pfont->data.loca = offset;
 	    loca_size = u32(tab + 12);
 	} else if (!memcmp(tab, "maxp", 4)) {
-	    const byte *maxp;
+	    byte maxp[30];
 
-	    ACCESS(offset, 30, maxp);
+	    READ_SFNTS(pfont, offset, 30, maxp);
 	    pfont->data.trueNumGlyphs = U16(maxp + 4);
 	} else if (!memcmp(tab, "name", 4)) {
 	    pfont->data.name_offset = offset;
 	} else if (!memcmp(tab, "vhea", 4)) {
-	    const byte *vhea;
+	    byte vhea[36];
 
-	    ACCESS(offset, 36, vhea);
+	    READ_SFNTS(pfont, offset, 36, vhea);
 	    pfont->data.metrics[1].numMetrics = U16(vhea + 34);
 	} else if (!memcmp(tab, "vmtx", 4)) {
 	    pfont->data.metrics[1].offset = offset;
@@ -229,7 +269,7 @@ gs_type42_font_init(gs_font_type42 * pfont, int subfontID)
     pfont->data.get_metrics = gs_type42_default_get_metrics;
 
     /* Now build the len_glyphs array since 'loca' may not be properly sorted */
-    pfont->data.len_glyphs = (uint *)gs_alloc_bytes(pfont->memory, pfont->data.numGlyphs * sizeof(uint),
+    pfont->data.len_glyphs = (uint *)gs_alloc_byte_array(pfont->memory, loca_size, sizeof(uint),
 						    "gs_type42_font");
     if (pfont->data.len_glyphs == 0)
 	return_error(gs_error_VMerror);
@@ -248,26 +288,37 @@ gs_type42_font_init(gs_font_type42 * pfont, int subfontID)
 	glyph_start = glyph_offset;
     }
     if (i < loca_size) {
-        uint j;
-	ulong trial_glyph_length;
         /*
-         * loca was out of order, build the len_glyphs the hard way      
-	 * Assume that some of the len_glyphs built so far may be wrong 
-	 * For each starting offset, find the next higher ending offset
-	 * Note that doing this means that there can only be zero length
-	 * glyphs that have loca table offset equal to the last 'dummy'
-         * entry. Otherwise we assume that it is a duplicated entry.
+         * loca was out of order, build the len_glyphs the hard way.      
+	 * For each glyph, we use the next higher or equal
+	 * glyph offset to compute the glyph length.
+	 * It assumes no overlapping and no duplicate glyphs.
 	 */
-	for (i = 0; i < loca_size - 1; i++) {
-	    glyph_start = get_glyph_offset(pfont, i);
-	    for (j = 1, glyph_length = 0x80000000; j < loca_size; j++) {
-		glyph_offset = get_glyph_offset(pfont, j);
-		trial_glyph_length = glyph_offset - glyph_start;
-		if ((trial_glyph_length > 0) && (trial_glyph_length < glyph_length))
-		    glyph_length = trial_glyph_length;
+	ulong last_glyph_offset = glyph_size;
+	gs_type42_font_init_sort_t *psort;
+	gs_type42_font_init_sort_t *psortary = 
+	    (gs_type42_font_init_sort_t *)gs_alloc_byte_array(pfont->memory, 
+		loca_size, sizeof(gs_type42_font_init_sort_t), "gs_type42_font_init(sort loca)");
+
+	if (psortary == 0)
+	    return_error(gs_error_VMerror);
+	for (i = 0, psort = psortary; i < loca_size; i++, psort++) {
+	    psort->glyph_num = i;
+	    psort->glyph_offset = get_glyph_offset(pfont, i);
 	    }
-	    pfont->data.len_glyphs[i] = glyph_length < 0x80000000 ? glyph_length : 0;
+	qsort(psortary, loca_size, sizeof(gs_type42_font_init_sort_t), gs_type42_font_init_compare);
+	if (psortary[loca_size - 1].glyph_offset > glyph_size)
+	    return_error(gs_error_invalidfont);
+	for (i = loca_size; i--;) {
+	    psort = psortary + i;
+	    pfont->data.len_glyphs[psort->glyph_num] = last_glyph_offset - psort->glyph_offset;
+	    last_glyph_offset = psort->glyph_offset;
 	}
+	/* Well the last element of len_glyphs is never used.
+	   We compute it because we're interesting whether it is not zero sometimes. 
+	   To know that, set a conditional breakpoint at the next statement.
+	 */
+	gs_free_object(pfont->memory, psortary, "gs_type42_font_init(sort loca)");
     }
     /*
      * If the font doesn't have a valid FontBBox, compute one from the
@@ -483,24 +534,15 @@ default_get_outline(gs_font_type42 * pfont, uint glyph_index,
 	     * Perhaps we can handle it (with a low performance),
 	     * making a contiguous copy.
 	     */
-	    uint left = glyph_length;
 
 	    /* 'code' is the returned length */
 	    buf = (byte *)gs_alloc_string(pgd->memory, glyph_length, "default_get_outline");
 	    if (buf == 0)
 		return_error(gs_error_VMerror);
 	    gs_glyph_data_from_string(pgd, buf, glyph_length, (gs_font *)pfont);
-	    for (;;) {
-		memcpy(buf + glyph_length - left, data, code);
-		if (!(left -= code))
-		    return 0;
-		code = (*string_proc)(pfont, (ulong)(pfont->data.glyf + glyph_start + 
-		              glyph_length - left), left, &data);
-		if (code < 0) 
-		    return code;
-		if (code == 0) 
-		    code = left;
-	    }
+	    memcpy(buf, data, code);
+	    return gs_type42_read_data(pfont, pfont->data.glyf + glyph_start + code, 
+				       glyph_length - code, buf + code);
 	}
     }
     return 0;
@@ -736,8 +778,6 @@ private int
 simple_glyph_metrics(gs_font_type42 * pfont, uint glyph_index, int wmode,
 		     float sbw[4])
 {
-    int (*string_proc)(gs_font_type42 *, ulong, uint, const byte **) =
-	pfont->data.string_proc;
     double factor = 1.0 / pfont->data.unitsPerEm;
     uint width;
     int lsb;
@@ -746,24 +786,24 @@ simple_glyph_metrics(gs_font_type42 * pfont, uint glyph_index, int wmode,
     {
 	const gs_type42_mtx_t *pmtx = &pfont->data.metrics[wmode];
 	uint num_metrics = pmtx->numMetrics;
-	const byte *pmetrics;
+	byte pmetrics[4];
 
 	if (pmtx->length == 0)
 	    return_error(gs_error_rangecheck);
 	if (glyph_index < num_metrics) {
-	    ACCESS(pmtx->offset + glyph_index * 4, 4, pmetrics);
+	    READ_SFNTS(pfont, pmtx->offset + glyph_index * 4, 4, pmetrics);
 	    width = U16(pmetrics);
 	    lsb = S16(pmetrics + 2);
 	} else {
 	    uint offset = pmtx->offset + num_metrics * 4;
 	    uint glyph_offset = (glyph_index - num_metrics) * 2;
-	    const byte *plsb;
+	    byte plsb[2];
 
-	    ACCESS(offset - 4, 4, pmetrics);
+	    READ_SFNTS(pfont, offset - 4, 4, pmetrics);
 	    width = U16(pmetrics);
 	    if (glyph_offset >= pmtx->length)
 		glyph_offset = pmtx->length - 2;
-	    ACCESS(offset + glyph_offset, 2, plsb);
+	    READ_SFNTS(pfont, offset + glyph_offset, 2, plsb);
 	    lsb = S16(plsb);
 	}
     }
@@ -1208,17 +1248,17 @@ private int get_from_names_table(gs_font_type42 *pfont, gs_font_info_t *info,
 {
     int (*string_proc)(gs_font_type42 *, ulong, uint, const byte **) =
 	pfont->data.string_proc;
-    const byte *t;
+    byte t[12];
     ushort num_records, strings_offset, i, language_id = 0xffff, length0 = 0, offset0 = 0;
     int code;
 
-    ACCESS(pfont->data.name_offset + 2, 4, t);
+    READ_SFNTS(pfont, pfont->data.name_offset + 2, 4, t);
     num_records = U16(t);
     strings_offset = U16(t + 2);
     for (i = 0; i < num_records; i++) {
 	ushort platformID, specificID, languageID, nameID, length, offset;
 
-	ACCESS(pfont->data.name_offset + 6 + i * 12, 12, t);
+	READ_SFNTS(pfont, pfont->data.name_offset + 6 + i * 12, 12, t);
 	platformID = U16(t + 0);
 	specificID = U16(t + 2);
 	languageID = U16(t + 4);
@@ -1238,8 +1278,9 @@ private int get_from_names_table(gs_font_type42 *pfont, gs_font_info_t *info,
     }
     if (language_id == 0xffff)
 	return 0;
-    ACCESS(pfont->data.name_offset + strings_offset + offset0, length0, t);
-    pmember->data = t;
+    if ((*string_proc)(pfont, pfont->data.name_offset + strings_offset + offset0, 
+				length0, &pmember->data) != 0)
+	return_error(gs_error_invalidfont);
     pmember->size = length0;
     info->members |= member;
     return 0;
