@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gsutil.c,v 1.8 2007/08/01 14:26:15 jemarch Exp $ */
+/* $Id: gsutil.c,v 1.10 2007/09/11 15:24:27 Arabidopsis Exp $ */
 /* Utilities for Ghostscript library */
 
 #include "string_.h"
@@ -303,21 +303,21 @@ gs_enable_object_tagging()
         BITTAG = GS_UNKNOWN_TAG;
 }
 
-
 void
 gs_set_object_tag(gs_state * pgs, const gs_object_tag_type_t tag)
 {
     if (BITTAG != GS_DEVICE_DOESNT_SUPPORT_TAGS) {
-	if ( BITTAG != tag ) {
-	    /* mkromfs breaks this dependance 
-	       NB: needs to be fixed.
-	    gx_unset_dev_color(pgs);
-	    **/
-	    BITTAG = tag;
-	    /* the assumption is made that the caller will:
-	     * gx_set_dev_color(pgs);
-	     */
-	}
+		if ( pgs->object_tag != tag ) {
+			pgs->object_tag = tag;
+			/* mkromfs breaks this dependance 
+			   NB: needs to be fixed.
+			gx_unset_dev_color(pgs);
+			**/
+			BITTAG = tag;
+			/* the assumption is made that the caller will:
+			 * gx_set_dev_color(pgs);
+			 */
+		}
     }
 }
 
