@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/*$Id: gxacpath.c,v 1.9 2007/08/01 14:26:15 jemarch Exp $ */
+/*$Id: gxacpath.c,v 1.11 2007/09/11 15:24:20 Arabidopsis Exp $ */
 /* Accumulator for clipping paths */
 #include "gx.h"
 #include "gserrors.h"
@@ -43,7 +43,7 @@ private dev_proc_pattern_manage(accum_pattern_manage);
 extern_st(st_clip_list);
 private
 ENUM_PTRS_WITH(device_cpath_accum_enum_ptrs, gx_device_cpath_accum *pdev)
-    if (index > st_device_max_ptrs)
+    if (index >= st_device_max_ptrs)
 	return ENUM_USING(st_clip_list, &pdev->list, sizeof(gx_clip_list), index - st_device_max_ptrs);
     ENUM_PREFIX(st_device, 0);
 ENUM_PTRS_END
