@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gxclfile.c,v 1.6 2007/08/01 14:26:17 jemarch Exp $ */
+/* $Id: gxclfile.c,v 1.8 2007/09/11 15:24:39 Arabidopsis Exp $ */
 /* File-based command list implementation */
 #include "stdio_.h"
 #include "string_.h"
@@ -33,7 +33,7 @@
 
 /* ------ Open/close/unlink ------ */
 
-int
+private int
 clist_fopen(char fname[gp_file_name_sizeof], const char *fmode,
 	    clist_file_ptr * pcf, gs_memory_t * mem, gs_memory_t *data_mem,
 	    bool ok_to_compress)
@@ -53,7 +53,13 @@ clist_fopen(char fname[gp_file_name_sizeof], const char *fmode,
     return 0;
 }
 
-int
+private int
+clist_unlink(const char *fname)
+{
+    return (unlink(fname) != 0 ? gs_note_error(gs_error_ioerror) : 0);
+}
+
+private int
 clist_fclose(clist_file_ptr cf, const char *fname, bool delete)
 {
     return (fclose((FILE *) cf) != 0 ? gs_note_error(gs_error_ioerror) :
@@ -61,21 +67,9 @@ clist_fclose(clist_file_ptr cf, const char *fname, bool delete)
 	    0);
 }
 
-int
-clist_unlink(const char *fname)
-{
-    return (unlink(fname) != 0 ? gs_note_error(gs_error_ioerror) : 0);
-}
-
 /* ------ Writing ------ */
 
-long
-clist_space_available(long requested)
-{
-    return requested;
-}
-
-int
+private int
 clist_fwrite_chars(const void *data, uint len, clist_file_ptr cf)
 {
     return fwrite(data, 1, len, (FILE *) cf);
@@ -83,7 +77,7 @@ clist_fwrite_chars(const void *data, uint len, clist_file_ptr cf)
 
 /* ------ Reading ------ */
 
-int
+private int
 clist_fread_chars(void *data, uint len, clist_file_ptr cf)
 {
     FILE *f = (FILE *) cf;
@@ -117,25 +111,25 @@ clist_fread_chars(void *data, uint len, clist_file_ptr cf)
 
 /* ------ Position/status ------ */
 
-int
+private int
 clist_set_memory_warning(clist_file_ptr cf, int bytes_left)
 {
     return 0;			/* no-op */
 }
 
-int
+private int
 clist_ferror_code(clist_file_ptr cf)
 {
     return (ferror((FILE *) cf) ? gs_error_ioerror : 0);
 }
 
-int64_t
+private int64_t
 clist_ftell(clist_file_ptr cf)
 {
     return gp_ftell_64((FILE *) cf);
 }
 
-void
+private void
 clist_rewind(clist_file_ptr cf, bool discard_data, const char *fname)
 {
     FILE *f = (FILE *) cf;
@@ -159,8 +153,29 @@ clist_rewind(clist_file_ptr cf, bool discard_data, const char *fname)
     }
 }
 
-int
+private int
 clist_fseek(clist_file_ptr cf, int64_t offset, int mode, const char *ignore_fname)
 {
     return gp_fseek_64((FILE *) cf, offset, mode);
 }
+
+private clist_io_procs_t clist_io_procs_file = {
+    clist_fopen,
+    clist_fclose,
+    clist_unlink,
+    clist_fwrite_chars,
+    clist_fread_chars,
+    clist_set_memory_warning,
+    clist_ferror_code,
+    clist_ftell,
+    clist_rewind,
+    clist_fseek,
+};
+
+init_proc(gs_gxclfile_init);
+int
+gs_gxclfile_init(gs_memory_t *mem)
+{
+    clist_io_procs_file_global = &clist_io_procs_file;
+    return 0;
+}
