@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gxclipm.c,v 1.8 2007/08/01 14:26:18 jemarch Exp $ */
+/* $Id: gxclipm.c,v 1.10 2007/09/11 15:23:58 Arabidopsis Exp $ */
 /* Mask clipping device */
 #include "memory_.h"
 #include "gx.h"
@@ -98,7 +98,8 @@ const gx_device_mask_clip gs_mask_clip_device =
   gx_forward_fill_linear_color_scanline,
   gx_forward_fill_linear_color_trapezoid,
   gx_forward_fill_linear_color_triangle,
-  gx_forward_update_spot_equivalent_colors
+  gx_forward_update_spot_equivalent_colors,
+  gx_forward_ret_devn_params
  }
 };
 
