@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gxclmem.c,v 1.6 2007/08/01 14:26:18 jemarch Exp $ */
+/* $Id: gxclmem.c,v 1.8 2007/09/11 15:24:33 Arabidopsis Exp $ */
 /* RAM-based command list implementation */
 #include "memory_.h"
 #include "gx.h"
@@ -137,7 +137,7 @@ LIMITATIONS.
    least as large as the fixed overhead of the compressor plus the
    decompressor, plus the expected compressed size of a block that size.
  */
-private const long COMPRESSION_THRESHOLD = 300000;
+private const long COMPRESSION_THRESHOLD = 32000000;
 
 #define NEED_TO_COMPRESS(f)\
   ((f)->ok_to_compress && (f)->total_space > COMPRESSION_THRESHOLD)
@@ -158,6 +158,8 @@ private_st_MEMFILE();
 	/* forward references */
 private void memfile_free_mem(MEMFILE * f);
 private int memfile_init_empty(MEMFILE * f);
+private int memfile_set_memory_warning(clist_file_ptr cf, int bytes_left);
+private int memfile_fclose(clist_file_ptr cf, const char *fname, bool delete);
 
 /************************************************/
 /*   #define DEBUG      /- force statistics -/  */
@@ -182,7 +184,7 @@ const byte *decomp_rd_ptr1, *decomp_rd_limit1;
 #endif
 
 /* ----------------------------- Memory Allocation --------------------- */
-void *	/* allocated memory's address, 0 if failure */
+private void *	/* allocated memory's address, 0 if failure */
 allocateWithReserve(
          MEMFILE  *f,			/* file to allocate mem to */
          int      sizeofBlock,		/* size of block to allocate */
@@ -224,7 +226,7 @@ allocateWithReserve(
 
 /* ---------------- Open/close/unlink ---------------- */
 
-int
+private int
 memfile_fopen(char fname[gp_file_name_sizeof], const char *fmode,
 	      clist_file_ptr /*MEMFILE * */  * pf,
 	      gs_memory_t *mem, gs_memory_t *data_mem, bool ok_to_compress)
@@ -325,7 +327,7 @@ finish:
     return code;
 }
 
-int
+private int
 memfile_fclose(clist_file_ptr cf, const char *fname, bool delete)
 {
     MEMFILE *const f = (MEMFILE *)cf;
@@ -361,7 +363,7 @@ memfile_fclose(clist_file_ptr cf, const char *fname, bool delete)
     return 0;
 }
 
-int
+private int
 memfile_unlink(const char *fname)
 {
     /*
@@ -374,7 +376,7 @@ memfile_unlink(const char *fname)
 /* ---------------- Writing ---------------- */
 
 /* Pre-alloc enough reserve mem blox to guarantee a write of N bytes will succeed */
-int	/* returns 0 ok, gs_error_VMerror if insufficient */
+private int	/* returns 0 ok, gs_error_VMerror if insufficient */
 memfile_set_memory_warning(clist_file_ptr cf, int bytes_left)
 {
     MEMFILE *const f = (MEMFILE *)cf;
@@ -617,7 +619,7 @@ memfile_next_blk(MEMFILE * f)
     return (ecode);
 }
 
-int	/* returns # of chars actually written */
+private int	/* returns # of chars actually written */
 memfile_fwrite_chars(const void *data, uint len, clist_file_ptr cf)
 {
     const char *str = (const char *)data;
@@ -830,7 +832,7 @@ memfile_get_pdata(MEMFILE * f)
 
 /* ---------------- Reading ---------------- */
 
-int
+private int
 memfile_fread_chars(void *data, uint len, clist_file_ptr cf)
 {
     char *str = (char *)data;
@@ -863,19 +865,19 @@ memfile_fread_chars(void *data, uint len, clist_file_ptr cf)
 
 /* ---------------- Position/status ---------------- */
 
-int
+private int
 memfile_ferror_code(clist_file_ptr cf)
 {
     return (((MEMFILE *) cf)->error_code);	/* errors stored here */
 }
 
-int64_t
+private int64_t
 memfile_ftell(clist_file_ptr cf)
 {
     return (((MEMFILE *) cf)->log_curr_pos);
 }
 
-void
+private void
 memfile_rewind(clist_file_ptr cf, bool discard_data, const char *ignore_fname)
 {
     MEMFILE *f = (MEMFILE *) cf;
@@ -891,7 +893,7 @@ memfile_rewind(clist_file_ptr cf, bool discard_data, const char *ignore_fname)
     }
 }
 
-int
+private int
 memfile_fseek(clist_file_ptr cf, int64_t offset, int mode, const char *ignore_fname)
 {
     MEMFILE *f = (MEMFILE *) cf;
@@ -1128,3 +1130,24 @@ memfile_init_empty(MEMFILE * f)
 
     return 0;
 }
+
+clist_io_procs_t clist_io_procs_memory = {
+    memfile_fopen,
+    memfile_fclose,
+    memfile_unlink,
+    memfile_fwrite_chars,
+    memfile_fread_chars,
+    memfile_set_memory_warning,
+    memfile_ferror_code,
+    memfile_ftell,
+    memfile_rewind,
+    memfile_fseek,
+};
+
+init_proc(gs_gxclmem_init);
+int
+gs_gxclmem_init(gs_memory_t *mem)
+{
+    clist_io_procs_memory_global = &clist_io_procs_memory;
+    return 0;
+}
