@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/*$Id: gxclrast.c,v 1.9 2007/08/01 14:26:19 jemarch Exp $ */
+/*$Id: gxclrast.c,v 1.11 2007/09/11 15:23:49 Arabidopsis Exp $ */
 /* Command list interpreter/rasterizer */
 #include "memory_.h"
 #include "gx.h"
@@ -54,6 +54,8 @@
 #include "gsserial.h"
 #include "gxdhtserial.h"
 #include "gzht.h"
+#include "gxshade.h"
+#include "gxshade4.h"
 
 extern_gx_device_halftone_list();
 extern_gx_image_type_table();
@@ -96,11 +98,29 @@ cmd_get_w(const byte * p, const byte ** rp)
     long val = *p++ & 0x7f;
     int shift = 7;
 
-    for (; val += (long)(*p & 0x7f) << shift, *p++ > 0x7f; shift += 7);
+    for (; val |= (long)(*p & 0x7f) << shift, *p++ > 0x7f; shift += 7);
     *rp = p;
     return val;
 }
 
+/* Get a variable-length fractional operand. */
+#define cmd_getfrac(var, p)\
+  BEGIN\
+    if ( !(*p & 1) ) var = (*p++) << 24;\
+    else { const byte *_cbp; var = cmd_get_frac31(p, &_cbp); p = _cbp; }\
+  END
+private frac31
+cmd_get_frac31(const byte * p, const byte ** rp)
+{
+    frac31 val = (*p++ & 0xFE) << 24;
+    int shift = 24 - 7;
+
+    for (; val |= (frac31)(*p & 0xFE) << shift, *p++ & 1; shift -= 7);
+    *rp = p;
+    return val;
+}
+
+
 /*
  * Define the structure for keeping track of the command reading buffer.
  *
@@ -297,6 +317,9 @@ clist_playback_band(clist_playback_action playback_action,
     int code = 0;
     ht_buff_t  ht_buff;
     gx_device *const orig_target = target;
+    gx_device_clip clipper_dev;
+    bool clipper_dev_open;
+    patch_fill_state_t pfs;
 
     cbuf.data = (byte *)cbuf_storage;
     cbuf.size = cbuf_size;
@@ -305,6 +328,7 @@ clist_playback_band(clist_playback_action playback_action,
     set_cb_end(&cbuf, cbuf.data + cbuf.size);
     cbp = cbuf.end;
 
+    pfs.dev = NULL; /* Indicate "not initialized". */
     memset(&ht_buff, 0, sizeof(ht_buff));
 
 in:				/* Initialize for a new page. */
@@ -312,6 +336,7 @@ in:				/* Initialize for a new page. */
     set_colors = state.colors;
     use_clip = false;
     pcpath = NULL;
+    clipper_dev_open = false;
     notes = sn_none;
     data_x = 0;
     {
@@ -950,14 +975,17 @@ set_phase:	/*
 			break;
 		    case cmd_opv_enable_clip:
 			pcpath = (use_clip ? &clip_path : NULL);
+			clipper_dev_open = false;
 			if_debug0('L', "\n");
 			break;
 		    case cmd_opv_disable_clip:
 			pcpath = NULL;
+			clipper_dev_open = false;
 			if_debug0('L', "\n");
 			break;
 		    case cmd_opv_begin_clip:
 			pcpath = NULL;
+			clipper_dev_open = false;
 			in_clip = true;
 			if_debug0('L', "\n");
 			code = gx_cpath_reset(&clip_path);
@@ -997,6 +1025,7 @@ set_phase:	/*
 				  cbox.q.y >= target_box.q.y);
 			}
 			pcpath = (use_clip ? &clip_path : NULL);
+			clipper_dev_open = false;
 			state.lop_enabled = clip_save.lop_enabled;
 			imager_state.log_op =
 			    (state.lop_enabled ? state.lop :
@@ -1390,6 +1419,124 @@ idata:			data_size = 0;
 			    code = clist_do_polyfill(tdev, ppath, &dev_color,
 						     imager_state.log_op);
 			    break;
+			case cmd_opv_fill_trapezoid:
+			    {
+				gs_fixed_edge left, right;
+				fixed ybot, ytop;
+				int options, swap_axes, wh;
+				fixed x0f;
+				fixed y0f;
+				gx_device *ttdev = tdev;
+
+				if (pcpath != NULL && !clipper_dev_open) {
+				    gx_make_clip_device(&clipper_dev, gx_cpath_list(pcpath)); /* fixme : create a global instance */
+				    clipper_dev.target = tdev;
+				    (*dev_proc(&clipper_dev, open_device))((gx_device *)&clipper_dev);
+				    clipper_dev_open = true;
+				}
+				if (clipper_dev_open)
+				    ttdev = (gx_device *)&clipper_dev;
+				cmd_getw(left.start.x, cbp);
+				cmd_getw(left.start.y, cbp);
+				cmd_getw(left.end.x, cbp);
+				cmd_getw(left.end.y, cbp);
+				cmd_getw(right.start.x, cbp);
+				cmd_getw(right.start.y, cbp);
+				cmd_getw(right.end.x, cbp);
+				cmd_getw(right.end.y, cbp);
+				cmd_getw(ybot, cbp);
+				cmd_getw(ytop, cbp);
+				cmd_getw(options, cbp);
+				swap_axes = options & 1;
+				wh = swap_axes ? tdev->width : tdev->height;
+				x0f = int2fixed(swap_axes ? y0 : x0);
+				y0f = int2fixed(swap_axes ? x0 : y0);
+				left.start.x -= x0f;
+				left.start.y -= y0f;
+				left.end.x -= x0f;
+				left.end.y -= y0f;
+				right.start.x -= x0f;
+				right.start.y -= y0f;
+				right.end.x -= x0f;
+				right.end.y -= y0f;
+				if (options & 2) {
+				    int num_components = tdev->color_info.num_components;
+				    frac31 c[4][GX_DEVICE_COLOR_MAX_COMPONENTS], *cc[4];
+				    byte colors_mask, i, j, m = 1;
+				    gs_fill_attributes fa;
+				    gs_fixed_rect clip;
+
+				    if (cbuf.end - cbp < 5 * cmd_max_intsize(sizeof(frac31)))
+					cbp = top_up_cbuf(&cbuf, cbp);
+				    cmd_getw(clip.p.x, cbp);
+				    cmd_getw(clip.p.y, cbp);
+				    cmd_getw(clip.q.x, cbp);
+				    cmd_getw(clip.q.y, cbp);
+				    clip.p.x -= x0f;
+				    clip.p.y -= y0f;
+				    clip.q.x -= x0f;
+				    clip.q.y -= y0f;
+				    fa.clip = &clip;
+				    fa.swap_axes = swap_axes;
+				    fa.ht = NULL;
+				    fa.lop = lop_default; /* fgixme: imager_state.log_op; */
+				    fa.ystart = ybot - y0f;
+				    fa.yend = ytop - y0f;
+				    cmd_getw(colors_mask, cbp);
+				    for (i = 0; i < 4; i++, m <<= 1) {
+					if (colors_mask & m) {
+					    if (cbuf.end - cbp < num_components * cmd_max_intsize(sizeof(frac31)))
+						cbp = top_up_cbuf(&cbuf, cbp);
+					    cc[i] = c[i];
+					    for (j = 0; j < num_components; j++)
+						cmd_getfrac(c[i][j], cbp);
+					} else
+					    cc[i] = NULL;
+				    }
+				    if (options & 4) {
+#					if 1 /* Disable to debug gx_fill_triangle_small. */
+					code = dev_proc(ttdev, fill_linear_color_triangle)(ttdev, &fa,
+							&left.start, &left.end, &right.start,
+							cc[0], cc[1], cc[2]);
+#					else
+					code = 0;
+#					endif
+					if (code == 0) {
+					    /* Fixme : The target device didn't fill the trapezoid and
+					       requests a decomposition. Call a code from gxshade6.c 
+					       for subdividing into smaller triangles : */
+					    if (pfs.dev == NULL)
+						code = gx_init_patch_fill_state_for_clist(tdev, &pfs, mem);
+					    if (code >= 0) {
+						pfs.dev = ttdev;
+						pfs.rect = clip; /* fixme: eliminate 'clip'. */
+						fa.pfs = &pfs;
+						code = gx_fill_triangle_small(ttdev, &fa,
+							    &left.start, &left.end, &right.start,
+							    cc[0], cc[1], cc[2]);
+					    }
+					}
+				    } else {
+					code = dev_proc(ttdev, fill_linear_color_trapezoid)(ttdev, &fa,
+							&left.start, &left.end, &right.start, &right.end,
+							cc[0], cc[1], cc[2], cc[3]);
+					if (code == 0) {
+					    /* Fixme : The target device didn't fill the trapezoid and
+					       requests a decomposition. 
+					       Currently we never call it with 4 colors (see gxshade6.c)
+					       and 2 colors must not return 0 - see comment to
+					       dev_t_proc_fill_linear_color_trapezoid in gxdevcli.c .
+					       Must not happen. */
+					    code = gs_note_error(gs_error_unregistered);
+					}
+				    }
+				} else
+				    code = gx_default_fill_trapezoid(ttdev, &left, &right,
+					max(ybot - y0f, fixed_half), 
+					min(ytop - y0f, int2fixed(wh)), swap_axes,
+					&dev_color, imager_state.log_op);
+			    }
+			   break;
 			default:
 			    goto bad_op;
 		    }
@@ -1532,13 +1679,18 @@ idata:			data_size = 0;
 	gs_free_object(target->memory, target, "gxclrast discard compositor");
 	target = orig_target;
     }
-    if (code < 0)
+    if (code < 0) {
+	if (pfs.dev != NULL)
+	    term_patch_fill_state(&pfs);
 	return_error(code);
+    }
     /* Check whether we have more pages to process. */
     if (playback_action != playback_action_setup && 
 	(cbp < cbuf.end || !seofp(s))
 	)
 	goto in;
+    if (pfs.dev != NULL)
+	term_patch_fill_state(&pfs);
     return code;
 }
 
@@ -2040,11 +2192,15 @@ out:
  * including the compositor name size, is smaller than the data buffer
  * size. This assumption is inherent in the basic design of the coding
  * and the de-serializer interface, as no length field is provided.
- * At the time of this writing, no compositor comes remotely close to
- * violating this assumption (largest create_compositor is about 16
- * bytes, while the command data buffer is 800 bytes), nor is any likely
- * to do so in the future. In the unlikely event that this assumption
- * is violated, a change in the encoding would be called for).
+ *
+ * At the time of this writing, no compositor violates this assumption.
+ * The largest create_compositor is currently 1275 bytes, while the command
+ * data buffer is 4096 bytes.
+ *
+ * In the event that this assumption is violated, a change in the encoding
+ * would be called for.
+ *
+ * See comment in gdevp14.c c_pdf14trans_read PDF14_BEGIN_TRANS_MASK case.
  */
 extern_gs_find_compositor();
 
@@ -2063,7 +2219,8 @@ read_create_compositor(
     gx_device *                 tdev = *ptarget;
 
     /* fill the command buffer (see comment above) */
-    cbp = top_up_cbuf(pcb, cbp);
+	if (pcb->end - cbp < MAX_CLIST_COMPOSITOR_SIZE + sizeof(comp_id))
+		cbp = top_up_cbuf(pcb, cbp);
 
     /* find the appropriate compositor method vector */
     comp_id = *cbp++;
@@ -2072,6 +2229,11 @@ read_create_compositor(
 
     /* de-serialize the compositor */
     code = pcomp_type->procs.read(&pcomp, cbp, pcb->end - cbp, mem);
+    
+    /* If we read more than the maximum expected, return a rangecheck error */
+    if ( code > MAX_CLIST_COMPOSITOR_SIZE )
+ 		return_error(gs_error_rangecheck);
+   	
     if (code > 0)
         cbp += code;
     pcb->ptr = cbp;
@@ -2467,8 +2629,6 @@ vhc:	    E = B + D, F = D = A + C, C = B, B = A, A = 0;
 
 /*
  * Execute a polyfill -- either a fill_parallelogram or a fill_triangle.
- * If we ever implement fill_trapezoid in the band list, that will be
- * detected here too.
  *
  * Note that degenerate parallelograms or triangles may collapse into
  * a single line or point.  We must check for this so we don't try to
