@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gxcmap.c,v 1.9 2007/08/01 14:26:19 jemarch Exp $ */
+/* $Id: gxcmap.c,v 1.11 2007/09/11 15:24:16 Arabidopsis Exp $ */
 /* Color mapping for Ghostscript */
 #include "gx.h"
 #include "gserrors.h"
@@ -371,12 +371,21 @@ gx_error_get_color_mapping_procs(const gx_device * dev)
 {
     /*
      * We should never get here.  If we do then we do not have a "get_color_mapping_procs"
-     * routine for the device.
+     * routine for the device. This will be noisy, but better than returning NULL which
+     * would lead to SEGV (Segmentation Fault) errors when this is used.
      */
-#ifdef DEBUG
-    dprintf("No get_color_mapping_procs proc defined for device.\n");
-#endif
-    return NULL;
+    eprintf1("No get_color_mapping_procs proc defined for device '%s'\n", dev->dname);
+    switch (dev->color_info.num_components) {
+      case 1:     /* DeviceGray or DeviceInvertGray */
+	return gx_default_DevGray_get_color_mapping_procs(dev);
+
+      case 3:
+	return gx_default_DevRGB_get_color_mapping_procs(dev);
+
+      case 4:
+      default:		/* Unknown color model - punt with CMYK */
+        return gx_default_DevCMYK_get_color_mapping_procs(dev);
+    }
 }
     
 /* ----- Default color component name to colorant index conversion routines ------ */
@@ -617,6 +626,21 @@ gx_remap_concrete_DGray(const frac * pconc, const gs_color_space * pcs,
 	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
 			gs_color_select_t select)
 {
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    int code = pcb->client_procs->remap_DeviceGray(pcb, pconc,
+						pcs, pdc, pis, dev, select);
+
+	    if (code == 0)
+		return 0;
+	}
+    }
+#endif
+
     if (pis->alpha == gx_max_color_value)
 	(*pis->cmap_procs->map_gray)
 	    (pconc[0], pdc, pis, dev, select);
@@ -638,6 +662,21 @@ gx_remap_DeviceGray(const gs_client_color * pc, const gs_color_space * pcs,
     pdc->ccolor.paint.values[0] = pc->paint.values[0];
     pdc->ccolor_valid = true;
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    int code = pcb->client_procs->remap_DeviceGray(pcb, &fgray,
+						pcs, pdc, pis, dev, select);
+
+	    if (code == 0)
+		return 0;
+	}
+    }
+#endif
+
     if (pis->alpha == gx_max_color_value)
 	(*pis->cmap_procs->map_gray)
 	    (fgray, pdc, pis, dev, select);
@@ -664,6 +703,20 @@ gx_remap_concrete_DRGB(const frac * pconc, const gs_color_space * pcs,
 	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
 		       gs_color_select_t select)
 {
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    int code = pcb->client_procs->remap_DeviceRGB(pcb, pconc,
+						pcs, pdc, pis, dev, select);
+
+	    if (code == 0)
+		return 0;
+	}
+    }
+#endif
     if (pis->alpha == gx_max_color_value)
 	gx_remap_concrete_rgb(pconc[0], pconc[1], pconc[2],
 			      pdc, pis, dev, select);
@@ -688,6 +741,25 @@ gx_remap_DeviceRGB(const gs_client_color * pc, const gs_color_space * pcs,
     pdc->ccolor.paint.values[2] = pc->paint.values[2];
     pdc->ccolor_valid = true;
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    frac conc[3];
+	    int code;
+
+	    conc[0] = fred;
+	    conc[1] = fgreen;
+	    conc[2] = fblue;
+	    code = pcb->client_procs->remap_DeviceRGB(pcb, conc,
+						pcs, pdc, pis, dev, select);
+	    if (code == 0)
+		return 0;
+	}
+    }
+#endif
     if (pis->alpha == gx_max_color_value)
 	gx_remap_concrete_rgb(fred, fgreen, fblue,
 			      pdc, pis, dev, select);
@@ -716,6 +788,20 @@ gx_remap_concrete_DCMYK(const frac * pconc, const gs_color_space * pcs,
 			gs_color_select_t select)
 {
 /****** IGNORE alpha ******/
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    int code = pcb->client_procs->remap_DeviceCMYK(pcb, pconc,
+						pcs, pdc, pis, dev, select);
+
+	    if (code == 0)
+		return 0;
+	}
+    }
+#endif
     gx_remap_concrete_cmyk(pconc[0], pconc[1], pconc[2], pconc[3], pdc,
 			   pis, dev, select);
     return 0;
@@ -735,6 +821,26 @@ gx_remap_DeviceCMYK(const gs_client_color * pc, const gs_color_space * pcs,
     pdc->ccolor.paint.values[3] = pc->paint.values[3];
     pdc->ccolor_valid = true;
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    frac conc[4];
+	    int code;
+
+	    conc[0] = (frac)unit_frac(pc->paint.values[0], ft0);
+	    conc[1] = (frac)unit_frac(pc->paint.values[1], ft1);
+	    conc[2] = (frac)unit_frac(pc->paint.values[2], ft2);
+	    conc[3] = (frac)unit_frac(pc->paint.values[3], ft3);
+	    code = pcb->client_procs->remap_DeviceCMYK(pcb, conc,
+						pcs, pdc, pis, dev, select);
+	    if (code == 0)
+		return 0;
+	}
+    }
+#endif
     gx_remap_concrete_cmyk((frac)unit_frac(pc->paint.values[0], ft0),
 			   (frac)unit_frac(pc->paint.values[1], ft1),
 			   (frac)unit_frac(pc->paint.values[2], ft2),
