@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gxcpath.c,v 1.9 2007/08/01 14:26:20 jemarch Exp $ */
+/* $Id: gxcpath.c,v 1.10 2007/09/11 15:24:04 Arabidopsis Exp $ */
 /* Implementation of clipping paths, other than actual clipping */
 #include "gx.h"
 #include "gserrors.h"
@@ -381,56 +381,66 @@ gx_cpath_path_list_new(gs_memory_t *mem, gx_clip_path *pcpath, int rule,
 
 /* ------ Clipping path accessing ------ */
 
+/* Synthesize a path from a clipping path. */
+int
+gx_cpath_to_path_synthesize(const gx_clip_path * pcpath, gx_path * ppath)
+{
+    /* Synthesize a path. */
+    gs_cpath_enum cenum;
+    gs_fixed_point pts[3];
+    int code;
+
+    gx_cpath_enum_init(&cenum, pcpath);
+    while ((code = gx_cpath_enum_next(&cenum, pts)) != 0) {
+	switch (code) {
+	    case gs_pe_moveto:
+		code = gx_path_add_point(ppath, pts[0].x, pts[0].y);
+		break;
+	    case gs_pe_lineto:
+		code = gx_path_add_line_notes(ppath, pts[0].x, pts[0].y,
+					   gx_cpath_enum_notes(&cenum));
+		break;
+	    case gs_pe_curveto:
+		code = gx_path_add_curve_notes(ppath, pts[0].x, pts[0].y,
+					       pts[1].x, pts[1].y,
+					       pts[2].x, pts[2].y,
+					   gx_cpath_enum_notes(&cenum));
+		break;
+	    case gs_pe_closepath:
+		code = gx_path_close_subpath_notes(ppath,
+					   gx_cpath_enum_notes(&cenum));
+		break;
+	    default:
+		if (code >= 0)
+		    code = gs_note_error(gs_error_unregistered);
+	}
+	if (code < 0)
+	    break;
+    }
+    return 0;
+}
+
 /* Return the path of a clipping path. */
 int
 gx_cpath_to_path(gx_clip_path * pcpath, gx_path * ppath)
 {
     if (!pcpath->path_valid) {
-	/* Synthesize a path. */
-	gs_cpath_enum cenum;
-	gs_fixed_point pts[3];
 	gx_path rpath;
 	int code;
 
 	gx_path_init_local(&rpath, pcpath->path.memory);
-	gx_cpath_enum_init(&cenum, pcpath);
-	while ((code = gx_cpath_enum_next(&cenum, pts)) != 0) {
-	    switch (code) {
-		case gs_pe_moveto:
-		    code = gx_path_add_point(&rpath, pts[0].x, pts[0].y);
-		    break;
-		case gs_pe_lineto:
-		    code = gx_path_add_line_notes(&rpath, pts[0].x, pts[0].y,
-					       gx_cpath_enum_notes(&cenum));
-		    break;
-		case gs_pe_curveto:
-		    code = gx_path_add_curve_notes(&rpath, pts[0].x, pts[0].y,
-						   pts[1].x, pts[1].y,
-						   pts[2].x, pts[2].y,
-					       gx_cpath_enum_notes(&cenum));
-		    break;
-		case gs_pe_closepath:
-		    code = gx_path_close_subpath_notes(&rpath,
-					       gx_cpath_enum_notes(&cenum));
-		    break;
-		default:
-		    if (code >= 0)
-			code = gs_note_error(gs_error_unregistered);
-	    }
-	    if (code < 0)
-		break;
-	}
-	if (code >= 0)
-	    code = gx_path_assign_free(&pcpath->path, &rpath);
+	code = gx_cpath_to_path_synthesize(pcpath, &rpath);
 	if (code < 0) {
 	    gx_path_free(&rpath, "gx_cpath_to_path error");
 	    return code;
 	}
+	code = gx_path_assign_free(&pcpath->path, &rpath);
+	if (code < 0)
+	    return code;
 	pcpath->path_valid = true;
     }
     return gx_path_assign_preserve(ppath, &pcpath->path);
 }
-
 /* Return the inner and outer check rectangles for a clipping path. */
 /* Return true iff the path is a rectangle. */
 bool
@@ -480,7 +490,7 @@ gx_cpath_list(const gx_clip_path *pcpath)
 }
 /* Internal non-const version of the same accessor. */
 inline private gx_clip_list *
-gx_cpath_list_private(gx_clip_path *pcpath)
+gx_cpath_list_private(const gx_clip_path *pcpath)
 {
     return &pcpath->rect_list->list;
 }
@@ -718,6 +728,8 @@ gx_cpath_scale_exp2_shared(gx_clip_path * pcpath, int log2_scale_x,
 		SCALE_V(ymax, log2_scale_y);
 #undef SCALE_V
 	    }
+	list->xmin = arith_rshift(list->xmin, -log2_scale_x);
+	list->xmax = arith_rshift(list->xmax, -log2_scale_x);
     }
     pcpath->id = gs_next_ids(pcpath->path.memory, 1);	/* path changed => change id */
     return 0;
@@ -766,7 +778,7 @@ gx_clip_list_from_rectangle(register gx_clip_list * clp,
 
 /* Start enumerating a clipping path. */
 int
-gx_cpath_enum_init(gs_cpath_enum * penum, gx_clip_path * pcpath)
+gx_cpath_enum_init(gs_cpath_enum * penum, const gx_clip_path * pcpath)
 {
     if ((penum->using_path = pcpath->path_valid)) {
 	gx_path_enum_init(&penum->path_enum, &pcpath->path);
@@ -1020,6 +1032,40 @@ gx_cpath_rect_visible(gx_clip_path * pcpath, gs_int_rect *prect)
     return false;
 }
 
+int
+gx_cpath_copy(const gx_clip_path * from, gx_clip_path * pcpath)
+{   /* *pcpath must be initialized. */
+    gx_clip_rect *r, *s;
+    gx_clip_list *l = &pcpath->rect_list->list;
+
+    pcpath->path_valid = false;
+    /* NOTE: pcpath->path still contains the old path. */
+    if (pcpath->path_list)
+	rc_decrement(pcpath->path_list, "gx_cpath_copy");
+    pcpath->path_list = NULL;
+    pcpath->rule = from->rule;
+    pcpath->outer_box = from->outer_box;
+    pcpath->inner_box = from->inner_box;
+    l->single = from->rect_list->list.single;
+    for (r = from->rect_list->list.head; r != NULL; r = r->next) {
+	s = gs_alloc_struct(from->rect_list->rc.memory, gx_clip_rect, &st_clip_rect, "gx_cpath_copy");
+	if (s == NULL)
+	    return_error(gs_error_VMerror);
+	*s = *r;
+	s->next = NULL;
+	if (l->tail) {
+	    s->prev = l->tail;
+	    l->tail->next = s;
+	} else {
+	    l->head = s;
+	    s->prev = NULL;
+	}
+	l->tail = s;
+    }
+    l->count = from->rect_list->list.count;
+    return 0;
+}
+
 /* ------ Debugging printout ------ */
 
 #ifdef DEBUG
