@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gxdevcli.h,v 1.9 2007/08/01 14:26:21 jemarch Exp $ */
+/* $Id: gxdevcli.h,v 1.11 2007/09/11 15:24:34 Arabidopsis Exp $ */
 /* Definitions for device clients */
 
 #ifndef gxdevcli_INCLUDED
@@ -147,6 +147,10 @@ typedef struct gx_stroke_params_s gx_stroke_params;
 #  define gs_imager_state_DEFINED
 typedef struct gs_imager_state_s gs_imager_state;
 #endif
+#ifndef patch_fill_state_t_DEFINED
+#  define patch_fill_state_t_DEFINED
+typedef struct patch_fill_state_s  patch_fill_state_t;
+#endif
 
 /* We need an abstract type for the image enumeration state, */
 /* for begin[_typed]_image. */
@@ -500,7 +504,7 @@ typedef struct gx_device_color_info_s {
                              : "DeviceCMYK") )
 
 #define dci_std_polarity(nc)                    \
-    ( (nc) == 4 ? GX_CINFO_POLARITY_SUBTRACTIVE \
+    ( (nc) >= 4 ? GX_CINFO_POLARITY_SUBTRACTIVE \
                 : GX_CINFO_POLARITY_ADDITIVE )
 
       /*
@@ -638,15 +642,6 @@ dev_page_proc_install(gx_default_install);
 dev_page_proc_begin_page(gx_default_begin_page);
 dev_page_proc_end_page(gx_default_end_page);
 
-#if ENABLE_NAMED_COLOR_CALLBACK
-/* Ptr to named color callback struct */
-#define NAMED_COLOR_PTR void * named_color_callback;
-#define INIT_NAMED_COLOR_PTR   NULL,		/* Initial value */
-#else
-#define NAMED_COLOR_PTR
-#define INIT_NAMED_COLOR_PTR
-#endif
-
 /* ---------------- Device structure ---------------- */
 
 /*
@@ -719,7 +714,8 @@ typedef struct gx_device_cached_colors_s {
 	bool IgnoreNumCopies;		/* if true, force num_copies = 1 */\
 	bool UseCIEColor;		/* for PS LL3 */\
 	bool LockSafetyParams;		/* If true, prevent unsafe changes */\
-	NAMED_COLOR_PTR			/* Pointer to named color callback struct */\
+	long band_offset_x;		/* offsets of clist band base to (mem device) buffer */\
+	long band_offset_y;		/* for rendering that is phase sensitive (wtsimdi) */\
 	gx_page_device_procs page_procs;	/* must be last */\
 		/* end of std_device_body */\
 	gx_device_procs procs	/* object procedures */
@@ -1238,7 +1234,9 @@ typedef enum {
     pattern_manage__finish_accum,
     pattern_manage__load,
     pattern_manage__shading_area,
-    pattern_manage__is_cpath_accum
+    pattern_manage__is_cpath_accum,
+    pattern_manage__shfill_doesnt_need_path,
+    pattern_manage__handles_clip_path
 } pattern_manage_t;
 
 #define dev_t_proc_pattern_manage(proc, dev_t)\
@@ -1287,6 +1285,7 @@ typedef struct gs_fill_attributes_s {
       const gx_device_halftone *ht; /* Reserved for possible use in future. */
       gs_logical_operation_t lop; /* Reserved for possible use in future. */
       fixed ystart, yend; /* Only for X-independent gradients. Base coordinates of the gradient. */
+      patch_fill_state_t *pfs; /* For gx_fill_triangle_small. Clients must not change. */
 } gs_fill_attributes;
 
 /* Fill a linear color scanline. */
@@ -1307,12 +1306,12 @@ typedef struct gs_fill_attributes_s {
 /* [p0 : p1] - left edge, from bottom to top.
    [p2 : p3] - right edge, from bottom to top.
    The filled area is within Y-spans of both edges. */
-/* If either (c0 and c1) or (c2 and c3) may be NULL.
+/* Either (c0 and c1) or (c2 and c3) may be NULL.
    In this case the color doesn't depend on X (on Y if fa->swap_axes).
    In this case the base coordinates for the color gradient
    may be unequal to p0, p1, p2, p3, and must be provided/taken
    in/from fa->ystart, fa->yend. 
-   The rerurn value 0 is not allowed in this case. */
+   The return value 0 is not allowed in this case. */
 /* Return values : 
   1 - success;
   0 - Too big. The area isn't filled. The client must decompose the area.
@@ -1354,6 +1353,20 @@ typedef struct gs_fill_attributes_s {
 #define dev_proc_update_spot_equivalent_colors(proc)\
   dev_t_proc_update_spot_equivalent_colors(proc, gx_device)
 
+/*
+ * return a pointer to the devn_params section of a device.  Return NULL
+ * if this field is not present within the device.
+ */
+#ifndef gs_devn_params_DEFINED
+#define gs_devn_params_DEFINED
+typedef struct gs_devn_params_s gs_devn_params;
+#endif
+
+#define dev_t_proc_ret_devn_params(proc, dev_t)\
+  gs_devn_params * proc(dev_t *dev)
+#define dev_proc_ret_devn_params(proc)\
+  dev_t_proc_ret_devn_params(proc, gx_device)
+
 /* Define the device procedure vector template proper. */
 
 #define gx_device_proc_struct(dev_t)\
@@ -1417,6 +1430,7 @@ typedef struct gs_fill_attributes_s {
 	dev_t_proc_fill_linear_color_trapezoid((*fill_linear_color_trapezoid), dev_t); \
 	dev_t_proc_fill_linear_color_triangle((*fill_linear_color_triangle), dev_t); \
 	dev_t_proc_update_spot_equivalent_colors((*update_spot_equivalent_colors), dev_t); \
+	dev_t_proc_ret_devn_params((*ret_devn_params), dev_t); \
 }
 
 
