@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gxdevmem.h,v 1.7 2007/08/01 14:26:21 jemarch Exp $ */
+/* $Id: gxdevmem.h,v 1.8 2007/09/11 15:24:09 Arabidopsis Exp $ */
 /* Structure and procedures for memory devices */
 /* Requires gxdevice.h */
 
@@ -183,17 +183,17 @@ extern_st(st_device_memory);
  * size includes both the bitmap and the line pointers.
  */
 /* bits only */
-ulong gdev_mem_bits_size(const gx_device_memory *mdev, int width,
-			 int height);
+int gdev_mem_bits_size(const gx_device_memory *mdev, int width,
+			 int height, ulong *size);
 /* line pointers only */
 ulong gdev_mem_line_ptrs_size(const gx_device_memory *mdev, int width,
 			      int height);
 /* bits + line pointers */
-ulong gdev_mem_data_size(const gx_device_memory *mdev, int width,
-			 int height);
+int gdev_mem_data_size(const gx_device_memory *mdev, int width,
+			 int height, ulong *size);
 
-#define gdev_mem_bitmap_size(mdev)\
-  gdev_mem_data_size(mdev, (mdev)->width, (mdev)->height)
+#define gdev_mem_bitmap_size(mdev, size)\
+  gdev_mem_data_size(mdev, (mdev)->width, (mdev)->height, size)
 
 /*
  * Do the inverse computation: given the device width and a buffer size,
