@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gxdhtserial.c,v 1.8 2007/08/01 14:26:21 jemarch Exp $ */
+/* $Id: gxdhtserial.c,v 1.9 2007/09/11 15:24:30 Arabidopsis Exp $ */
 /* Serialization and de-serialization for (traditional) halftones */
 
 #include "memory_.h"
@@ -308,15 +308,18 @@ gx_ht_read_component_wts(gx_ht_order_component *pcomp,
     int hdr_size = wts_size(ws);
     int cell_size = ws->cell_width * ws->cell_height *
 	sizeof(wts_screen_sample_t);
+    int bufsize = 1+hdr_size+cell_size;
 
     memset(&pcomp->corder, 0, sizeof(pcomp->corder));
 
-    pcomp->corder.wts = gs_wts_from_buf(data);
+    if (size < bufsize)
+	return -1;
+    pcomp->corder.wts = gs_wts_from_buf(data, bufsize);
     pcomp->cname = 0;
     if (pcomp->corder.wts == NULL)
 	return -1;
 
-    return 1 + hdr_size + cell_size;
+    return bufsize;
 }
 
 /*
