@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gximag3x.c,v 1.9 2007/08/01 14:26:25 jemarch Exp $ */
+/* $Id: gximag3x.c,v 1.11 2007/09/11 15:24:23 Arabidopsis Exp $ */
 /* ImageType 3x image implementation */
 /****** THE REAL WORK IS NYI ******/
 #include "math_.h"		/* for ceil, floor */
@@ -495,6 +495,9 @@ make_midx_default(gx_device **pmidev, gx_device *dev, int width, int height,
     gx_device_memory *midev;
     int code;
 
+    if (width != 0)
+	if (height > max_ulong/width)	/* protect against overflow in bitmap size */
+	    return_error(gs_error_VMerror);
     if (mdproto == 0)
 	return_error(gs_error_rangecheck);
     midev = gs_alloc_struct(mem, gx_device_memory, &st_device_memory,
@@ -747,10 +750,13 @@ gx_image3x_plane_data(gx_image_enum_common_t * info,
 	      h, (mask_plane[0].data ? "+" : ""), penum->mask[0].y,
 	      (mask_plane[1].data ? "+" : ""), penum->mask[1].y,
 	      (pixel_planes[0].data ? "+" : ""), penum->pixel.y);
-    if (penum->mask[0].y >= penum->mask[0].height &&
-	penum->mask[1].y >= penum->mask[1].height &&
-	penum->pixel.y >= penum->pixel.height)
-	return 1;
+    if (penum->mask[0].depth == 0 || penum->mask[0].y >= penum->mask[0].height) {
+        if (penum->mask[1].depth == 0 || penum->mask[1].y >= penum->mask[1].height) {
+	    if (penum->pixel.y >= penum->pixel.height) {
+	        return 1;
+            }
+        }
+    }
     /*
      * The mask may be complete (gx_image_plane_data_rows returned 1),
      * but there may still be pixel rows to go, so don't return 1 here.
