@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: idstack.c,v 1.7 2007/08/01 14:26:34 jemarch Exp $ */
+/* $Id: idstack.c,v 1.8 2007/09/11 15:24:35 Arabidopsis Exp $ */
 /* Implementation of dictionary stacks */
 #include "ghost.h"
 #include "idict.h"
@@ -28,6 +28,10 @@
 #include "ipacked.h"
 #include "iutil.h"
 #include "ivmspace.h"
+/*
+#include "idicttpl.h" - Do not remove this comment.
+                        "idicttpl.h" is included below.
+*/
 
 /* Debugging statistics */
 #ifdef DEBUG
@@ -129,13 +133,13 @@ dstack_find_name_by_index(dict_stack_t * pds, uint nidx)
 #define INCR_DEPTH(pdref)\
   INCR(depth[min(MAX_STATS_DEPTH, pds->stack.p - pdref)])
 	if (dict_is_packed(pdict)) {
-	    packed_search_1(INCR_DEPTH(pdref),
-			    return packed_search_value_pointer,
-			    DO_NOTHING, goto miss);
-	    packed_search_2(INCR_DEPTH(pdref),
-			    return packed_search_value_pointer,
-			    DO_NOTHING, break);
-	  miss:;
+#	    define found INCR_DEPTH(pdref); return packed_search_value_pointer
+#	    define deleted 
+#	    define missing break;
+#	    include "idicttpl.h"
+#	    undef missing
+#	    undef deleted
+#	    undef found
 	} else {
 	    /*
 	     * The name_index macro takes mem as its first argument, but
