@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2007 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: imainarg.c,v 1.11 2007/08/01 14:26:36 jemarch Exp $ */
+/* $Id: imainarg.c,v 1.13 2007/09/11 15:23:59 Arabidopsis Exp $ */
 /* Command line parsing and dispatching */
 #include "ctype_.h"
 #include "memory_.h"
@@ -70,10 +70,6 @@ extern int zflushpage(i_ctx_t *);
 #  define GS_MAX_LIB_DIRS 25
 #endif
 
-#ifndef GS_BUG_MAILBOX
-#  define GS_BUG_MAILBOX "bug-ghostscript@gnu.org"
-#endif
-
 #define MAX_BUFFERED_SIZE 1024
 
 /* Note: sscanf incorrectly defines its first argument as char * */
@@ -200,7 +196,7 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
 	    (char *)gs_alloc_bytes(minst->heap, len, "GS_OPTIONS");
 
 	    gp_getenv(GS_OPTIONS, opts, &len);	/* can't fail */
-	    if (arg_push_memory_string(&args, opts, minst->heap))
+	    if (arg_push_memory_string(&args, opts, false, minst->heap))
 		return e_Fatal;
 	}
     }
@@ -372,7 +368,7 @@ run_stdin:
 		    char *p = arg_copy(arg, minst->heap);
 		    if (p == NULL)
 			return e_Fatal;
-		    arg_push_string(pal, p);
+		    arg_push_string(pal, p, true);
 		}
 		pal->expand_ats = ats;
 		break;
@@ -889,7 +885,7 @@ private const char help_usage2[] = "\
                                          embed %d or %ld for page #\n";
 private const char help_trailer[] = "\
 For more information, see %s.\n\
-Report bugs to %s, using the form in Bug-form.htm.\n";
+Please report bugs to bug-ghostscript@gnu.org.\n";
 private const char help_devices[] = "Available devices:";
 private const char help_default_device[] = "Default output device:";
 private const char help_emulators[] = "Input formats:";
@@ -1069,5 +1065,5 @@ print_help_trailer(const gs_main_instance *minst)
     if (gp_file_name_combine(gs_doc_directory, strlen(gs_doc_directory), 
 	    use_htm, strlen(use_htm), false, buffer, &blen) != gp_combine_success)
 	p = use_htm;
-    outprintf(minst->heap, help_trailer, p, GS_BUG_MAILBOX);
+    outprintf(minst->heap, help_trailer, p);
 }
