@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: iscannum.h,v 1.6 2007/08/01 14:26:38 jemarch Exp $ */
+/* $Id: iscannum.h,v 1.8 2007/09/11 15:24:19 Arabidopsis Exp $ */
 /* Interface to Ghostscript number scanner */
 
 #ifndef iscannum_INCLUDED
@@ -29,6 +29,6 @@
  * Note that scan_number does not mark the result ref as "new".
  */
 int scan_number(const byte * sp, const byte * end, int sign, ref * pref,
-		const byte ** psp, const bool PDFScanRules);
+		const byte ** psp, int scanner_options);
 
 #endif /* iscannum_INCLUDED */
