@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: scfd.c,v 1.7 2007/08/01 14:26:41 jemarch Exp $ */
+/* $Id: scfd.c,v 1.9 2007/09/11 15:23:45 Arabidopsis Exp $ */
 /* CCITTFax decoding filter */
 #include "stdio_.h"		/* includes std.h */
 #include "memory_.h"
@@ -52,7 +52,7 @@ s_CFD_init(stream_state * st)
     /* Because skip_white_pixels can look as many as 4 bytes ahead, */
     /* we need to allow 4 extra bytes at the end of the row buffers. */
     ss->lbuf = gs_alloc_bytes(st->memory, raster + 4, "CFD lbuf");
-    memset(ss->lbuf, white, raster);
+    memset(ss->lbuf, white, raster + 4);  /* + 4 for Valgrind */
     ss->lprev = 0;
     if (ss->lbuf == 0)
 	return ERRC;		/****** WRONG ******/
@@ -61,7 +61,7 @@ s_CFD_init(stream_state * st)
 	if (ss->lprev == 0)
 	    return ERRC;	/****** WRONG ******/
 	/* Clear the initial reference line for 2-D encoding. */
-	memset(ss->lprev, white, raster);
+	memset(ss->lprev, white, raster + 4); /* + 4 for Valgrind */
 	/* Ensure that the scan of the reference line will stop. */
 	ss->lprev[raster] = 0xa0;
     }
@@ -306,7 +306,7 @@ s_CFD_process(stream_state * st, stream_cursor_read * pr,
 	 * in the string of initial zeros in the EOL.  If EndOfLine
 	 * is false, we aren't sure what we should do....
 	 */
-	if (ss->EncodedByteAlign & !ss->EndOfLine)
+	if (ss->EncodedByteAlign && !ss->EndOfLine)
 	    ss->bits_left &= ~7;
     }
     /* If we're between scan lines, scan for EOLs. */
