@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: zbfont.c,v 1.9 2007/08/01 14:26:48 jemarch Exp $ */
+/* $Id: zbfont.c,v 1.11 2007/09/11 15:23:49 Arabidopsis Exp $ */
 /* Font creation utilities */
 #include "memory_.h"
 #include "string_.h"
@@ -40,8 +40,25 @@
 #include "istruct.h"
 #include "store.h"
 
-/* Structure descriptor */
+/* Structure descriptor and GC procedures for font_data */
 public_st_font_data();
+private
+CLEAR_MARKS_PROC(font_data_clear_marks)
+{
+    ref_struct_clear_marks(cmem, vptr, offset_of(font_data, u.type42.mru_sfnts_index)/*size*/, pstype);
+}
+private
+ENUM_PTRS_BEGIN_PROC(font_data_enum_ptrs)
+{
+    return ref_struct_enum_ptrs(mem, vptr, offset_of(font_data, u.type42.mru_sfnts_index)/*size*/, index, pep, pstype, gcst);
+}
+ENUM_PTRS_END_PROC
+private
+RELOC_PTRS_BEGIN(font_data_reloc_ptrs)
+{
+    ref_struct_reloc_ptrs(vptr, offset_of(font_data, u.type42.mru_sfnts_index)/*size*/, pstype, gcst);
+}
+RELOC_PTRS_END
 
 /* <string|name> <font_dict> .buildfont3 <string|name> <font> */
 /* Build a type 3 (user-defined) font. */
