@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: zfcid1.c,v 1.8 2007/08/01 14:26:50 jemarch Exp $ */
+/* $Id: zfcid1.c,v 1.10 2007/09/11 15:24:20 Arabidopsis Exp $ */
 /* CIDFontType 1 and 2 operators */
 #include "memory_.h"
 #include "ghost.h"
@@ -101,7 +101,7 @@ z11_CIDMap_proc(gs_font_cid2 *pfont, gs_glyph glyph)
 	return prgnum->value.intval;
     default:			/* array type */
         code = string_array_access_proc(pfont->memory, pcidmap, 1, cid * gdbytes,
-					gdbytes, &data);
+					gdbytes, NULL, NULL, &data);
 
 	if (code < 0)
 	    return code;
@@ -285,7 +285,7 @@ zbuildfont11(i_ctx_t *i_ctx_p)
     if (MetricsCount & 1)	/* only allowable values are 0, 2, 4 */
 	return_error(e_rangecheck);
     code = dict_find_string(op, "File", &pfile);
-    if (code < 0 && code != e_dictfull)
+    if (code < 0 && code != e_undefined)
 	return code;
     if (code > 0) {
 	ref *file_table_pos, *a, v;
