@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,9 +17,10 @@
 
 */
 
-/* $Id: ztoken.c,v 1.8 2007/08/01 14:26:54 jemarch Exp $ */
+/* $Id: ztoken.c,v 1.10 2007/09/11 15:24:06 Arabidopsis Exp $ */
 /* Token reading operators */
 #include "string_.h"
+#include "stat_.h" /* get system header early to avoid name clash on Cygwin */
 #include "ghost.h"
 #include "oper.h"
 #include "dstack.h"		/* for dict_find_name */
@@ -328,11 +329,12 @@ ztoken_scanner_options(const ref *upref, int old_options)
 	const char *pname;
 	int option;
     } named_scanner_option_t;
-    static const named_scanner_option_t named_options[4] = {
+    static const named_scanner_option_t named_options[] = {
 	{"ProcessComment", SCAN_PROCESS_COMMENTS},
 	{"ProcessDSCComment", SCAN_PROCESS_DSC_COMMENTS},
 	{"PDFScanRules", SCAN_PDF_RULES},
-	{"PDFScanInvNum", SCAN_PDF_INV_NUM}
+	{"PDFScanInvNum", SCAN_PDF_INV_NUM},
+	{"PDFScanUnsigned", SCAN_PDF_UNSIGNED}
     };
     int options = old_options;
     int i;
