@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
   This file is part of GNU ghostscript
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: ztrans.c,v 1.9 2007/08/01 14:26:54 jemarch Exp $ */
+/* $Id: ztrans.c,v 1.11 2007/09/11 15:23:55 Arabidopsis Exp $ */
 /* Transparency operators */
 #include "string_.h"
 #include "memory_.h"
@@ -37,6 +37,8 @@
 #include "iname.h"
 #include "store.h"
 #include "gsdfilt.h"
+#include "gdevdevn.h"
+#include "gxblend.h"
 #include "gdevp14.h"
 
 /* ------ Utilities ------ */
